""" Control SAFIR calculations

Notes
-----
The control module support SAFIR calculations
"""

#############
## IMPORTS ##
#############

import os
import subprocess
import shutil
import json
import time

from magnelPy.admin import TimeStamp


#####################
## REFERENCE PATHS ##
#####################

pathfile_path ="C:\\Users\\rvcoile\\Documents\\paths_SAFIRshell.json"

with open(pathfile_path) as json_file:
    paths = json.load(json_file)

## system path SAFIR executable ##
SAFIRpath=paths["SAFIRpath"]
Temp_Dir=paths["Temp_Dir"]



############
## MODULE ##
############

def runSAFIR():
	## *.in file with extension ##
	infile=SAFIRin_userInput()

	## testing ##
	print(infile)
	
	## execution ##
	SAFIR_run(infile)

def SAFIR_run(file,path=SAFIRpath,SW_removeItem=True,Temp_Dir=Temp_Dir):
	# run SAFIR *.in file
	#	copy *.tem file to Python directory in case of structural analysis

	curDir=os.getcwd()
	refDir='\\'.join(file.split('\\')[0:-1]) # path to folder of *.in file

	## check type of SAFIR calcuation
	SAFIRtype=SAFIR_type(file)

	# create working directory
	worker,workDir=WorkDirCreate(Temp_Dir,file)
	os.chdir(workDir)

	## Run calculation ##
	## if Thermal2D ==> run calculation
	if SAFIRtype=='Thermal2D':
		pass
	else: ## STRUCTURAL UNDER DEVELOPMENT ##
		## move *.tem file to SAFIRpy working directory
		temfilepath,temname=SAFIR_TEMinIN(file) # determine *.tem file (full path - *.in directory assumed)
		temtargetpath=workDir+'\\'+temname
		shutil.copy(temfilepath,temtargetpath) # copy *.tem file to SAFIRpy working directory
		
	# run SAFIR and move results to original directory
	SAFIR_exe(path,worker) # run SAFIR
	os.chdir(curDir)
	WorkDirClean(workDir,refDir)



#########
## AUX ##
#########

def WorkDirClean(WorkDir,refDir):
	files=os.listdir(WorkDir)
	for file in files:
		infile=WorkDir+'\\'+file
		try: shutil.move(infile,refDir)
		except : pass
	shutil.rmtree(WorkDir)
	# shutil.copytree(WorkDir,refDir)
	# shutil.move(WorkDir,refDir)


def WorkDirCreate(Temp_Dir,file):
	## create working dir and copy files to perform calculation ##
	sub=TimeStamp()
	workDir=Temp_Dir+"\\"+sub
	os.mkdir(workDir)

	name=file.split('\\')[-1]
	worker=workDir+"\\"+name
	shutil.copy(file,worker)

	return worker,workDir

def SAFIR_TEMinIN(file):
	# returns name *.TEM file referenced in *.IN file

	with open(file) as f:
		line=f.readline()
		while len(line)>0:
			line=f.readline() # read line
			if '.tem' in line:
				line=line.rstrip() # remove newline character
				infolderpath='\\'.join(file.split('\\')[0:-1]) # path to folder of *.in file
				temfilepath=infolderpath+'\\'+line
				return temfilepath,line # returns full path and name only of *.tem file

def SAFIR_exe(path,file):

	## remove *.in extension
	file=file[0:-3] 

	## run through subprocess
	subprocess.call([path,file])

def SAFIR_type(file):
	# returns type of *.in file: Thermal2D, Structural2D, Structural3D

	with open(file,encoding="utf-8") as f:
		line=f.readline() # first line
		line=f.readline().rstrip() # second line and remove newline indication

	if line == 'Safir_Thermal_Analysis': return 'Thermal2D'
	if line == 'Safir_Static_2D_Analysis': return 'Structural2D'
	if line == 'Safir_Static_3D_Analysis': return 'Structural3D'

def SAFIRin_userInput():

	## filename
	filename=input("\nPlease provide path to input file (*.in): ")
	if filename[0]=="\"": filename=filename[1:-1] # strips quotes from path

	return filename

#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__": 

	runSAFIR()



"""MaterialProperties\\SFRM
Notes
-----
Temperature-dependent properties for use in SFE.

Probabilitic material properties for SFRM at elevated temperatures
Ref(A): Gernay, T., Khorasani, N.E. and Garlock, M., 2016. Fire fragility curves for steel buildings
in a community context: A methodology. Engineering Structures, 113, pp.259-276.
"""

#############
## IMPORTS ##
#############

import numpy as np

############
## MODULE ##
############

def rho(theta,eps=0):
	""" SFRM density according to Ref(A)
	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	eps: float
		Probabilistic parameter (standard normal distribution)
		Default: eps = 0 for mean values
	Returns
	-------
	res : float
		 density [kg/m3]
	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import SFRM as sfrm
	print(sfrm.rho(20))
	>>> 284.58
	"""
	return np.exp(-2.028 +7.83*(theta)**-0.0065 + 0.122*eps) # [kg/m3]

def cp(theta,eps=0):
	""" SFRM specific heat according to Ref(A);

	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	eps: float
		Probabilistic parameter (standard normal distribution)
		Default: eps = 0 for mean values

	Returns
	-------
	res : float or np.array
		 specific heat [J/kgK]

	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import SFRM as sfrm
	import numpy as np	
	theta=np.arange(100,500,100)
	cp=sfrm.cp(theta)
	print(cp)
	>>> [924.58, 1031.12, 1117.92, 1188.98]
	"""
	return 1700-np.exp(6.81 - 1.61*(10**-3)*theta + 0.44*(10**-6)*(theta**2) + 0.213*eps)

def ki(theta,eps=0):
	""" SFRM conductivity according to Ref(A)

	Parameters
	----------
	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	eps: float
		Probabilistic parameter (standard normal distribution)
		Default: eps = 0 for mean values

	Returns
	-------
	res : float or np.array
		 conductivity [W/mK]

	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import SFRM as sfrm
	import numpy as np	
	theta=np.arange(100,500,100)
	ki=sfrm.ki(theta)
	print(ki)
	>>> [0.0794, 0.0954, 0.1141, 0.1360]
	"""	
	return np.exp(-2.72 + 1.89*(10**-3)*theta - 0.195*(10**-6)*(theta**2) + 0.209*eps)








####################
## MODULE IMPORTS ##
####################

import numpy as np
import scipy.stats as stats
import sympy as sy
import pandas as pd
from copy import deepcopy

from magnelPy import statFunc
from magnelPy.admin import df_writeToExcel
from magnelPy.stochVar import createStochVar,fx


##############
## FUNCTION ##
##############

def LHS(N, K, RedCor = False, MidPoint = False, Legacy=False ):
	#------------------------------------------------------------------------------------------------------------
	# This function generates a Latin Hypercube consisting of N realizations of each of the K random variables
	#
	# Input:
	# 	N = the number of realizations/samples
	# 	K = the number of random variables
	# 	RedCor = switch the option to reduce spurious correlation on/off (True/False)
	#	MidPoint = switch the option to take the midpoint value of each interval (True/False)
	#	Legacy = switch to use old (slower) code - new code has been verified; legacy option as part of risk management
	#
	# Output:
	#	numpy.ndarray : NxK array with LHS realizations (r-values between 0 and 1)
	# 
	# Procedure based on:
	# Olsson A., Sandberg G. & Dahlblom O. (2013) On Latin Hypercube Sampling for structural reliability analysis
	#     Structural Safety 25, pp 47-68
	#
	# Wouter Botte - 2017
	#------------------------------------------------------------------------------------------------------------

	if not isinstance(N, int ):
		print("________________________________________________________________________________")
		print("ERROR: the number of realizations N should be an integer")
		print("________________________________________________________________________________")
		return None
	if not isinstance(K, int ):
		print("________________________________________________________________________________")
		print("ERROR: the number of variables K should be an integer")
		print("________________________________________________________________________________")
		return None

	# Generate the matrix P containing K columns with random permutations of 1,...,N
	matrix = [ [ i for i in range(1, N + 1) ] for j in range(K) ]
	matrix = [ np.random.permutation( matrix[i] ) for i in range( len(matrix) ) ]
	P = np.transpose(matrix)

	# Reduction of spurious correlation if necessary
	if RedCor:
		if N < K:
			print("________________________________________________________________________________")
			print("ERROR: The number of realizations N is less than the number of variables K")
			print("	   In case of spurious correlation reduction N should be higher than K")
			print("________________________________________________________________________________")
			return None
		else:
			if Legacy: Y = np.array([ [ stats.norm.ppf(float(P[i][j])/(N + 1)) for j in range(K) ] for i in range(N)])
			else: Y=statFunc.Finv_Normal(P/(N+1),0,1)
			covY = np.cov(np.transpose(Y))
			L = np.linalg.cholesky(covY)
			Ystar = np.dot(Y, np.transpose(np.linalg.inv(L)))
			if Legacy: Pstar = np.transpose(np.array([ [sorted(Ystar[:,i]).index(v) + 1 for v in Ystar[:,i]] for i in range(K)]))
			else: Pstar = Ystar.argsort(axis=0).argsort(axis=0)+1
			P = Pstar

	# Generate the matrix R of independent random numbers from the uniform distribution
	if MidPoint: R=0.5
	else: R = np.array(np.random.uniform(size = (N,K)))

	# Generate sampling plan S
	S = 1./N*(P-R)
	return S

def MonteCarlo(limitstate,ParameterDict,nMC,method='MCS'):
	#------------------------------------------------------------------------------------------------------------
	# Limit state evaluation - WIP
	#
	# Input:
	# 	varDict = Dictionary with function parameters (in function of distribution)
	#		'name' = name of parameter: 'X'
	#		'Dist' = Distribution type: one of the following:
	#				'N' : Normal
	#				'LN' : Lognormal
	#				'MixedLN' : Mixed Lognormal (probability weighted contributions of LN constituents)
	#				'G' : Gumbel
	#				'DET' : Deterministic
	#		'm' = mean value X - or pd.Series of constituent mean values
	#		's' = standard deviation X - or pd.Series of constituent standard deviations
	#		'P' = pd.Series with probabilities of constituents (cfr. MixedLN)
	#		'info' = further notes
	#	rArray = np.array (or commutable) with quantile realizations ri (0,1)
	#
	# Output:
	#	parameter realizations xi
	# rvcpy - 2017
	#------------------------------------------------------------------------------------------------------------
	# performs crude Monte Carlo simulation
	# input
	# * limitstate: symbolic limit state function
	# * ParameterDict: dictionary of all parameters, including probabilistic discription
	# * nMC: number of simulations
	# output
	# * array of limit state evaluations, DataFrame with parameter values and limit state valuation, DataFrame with random values

	## create MCS array random values
	symbolList=limitstate.atoms(sy.Symbol)
	nvar=len(symbolList)
	rMatrix=np.random.rand(nMC,nvar)

	## calculate MCS array random realizations
	xMatrix=np.zeros((nMC,nvar))
	# indexing in the ParameterDict - general indexing  
	indexDict={}
	for key in ParameterDict:
		indexDict[ParameterDict[key]['name']]=key
	# random realization per parameter
	varOrder=[]
	for i,X in enumerate(symbolList):
		# save order for printing lists
		var=X.name
		varOrder.append(var)
		# collect Dict (stochastic) variable
		localDict=ParameterDict[indexDict[var]]
		xMatrix[:,i]=ParameterRealization_r(localDict,rMatrix[:,i])

	## evaluate limit state
	limitstateEval = sy.lambdify(tuple(varOrder), limitstate, 'numpy')
	lsEvalList=LS_evaluation(limitstateEval,nvar,xMatrix)


	## output
	# outR first, before appending the varOrder...
	outR=pd.DataFrame(rMatrix,columns=varOrder)
	# set output - variable realizations + limit state evaluation
	lsEvalList=np.reshape(lsEvalList,(nMC,1))
	full=np.concatenate((xMatrix,lsEvalList),axis=1)
	varOrder.append('LS')
	outX=pd.DataFrame(full,columns=varOrder)

	return lsEvalList, outX, outR

def MCS_var(varDict,n):
	#------------------------------------------------------------------------------------------------------------
	# Return n realizations of stochastic variable
	#
	# Input:
	# 	varDict = Dictionary with function parameters (in function of distribution) - cfr. standard format
	#	n = number of realizations
	# Output:
	#	pd.Series of realizations
	# concept Generaterandom.py TTH 11.2018
	#------------------------------------------------------------------------------------------------------------

	r=np.random.rand(n) # ri-values (random quantiles)
	return pd.Series(ParameterRealization_r(varDict,r),np.arange(n))


def Taylor(limitstate,ParameterDict):
	# Taylor approximation of mean value and standard deviation
	# input
	# * limitstate: symbolic limit state function
	# * ParameterDict: dictionary of all parameters, including probabilistic discription
	# output
	# * mean value, standard deviation

	# ^cfr. rvcpy - probabcalc.py 2017-11-24: created

	# list of symbols in limitstate
	symbolList=limitstate.atoms(sy.Symbol)
	nvar=len(symbolList)
	xMatrix=np.zeros((2,nvar)) 	# array mean values and standard deviations

	# indexing in the ParameterDict - general indexing  
	indexDict={}
	for key in ParameterDict:
		indexDict[ParameterDict[key]['name']]=key

	# values per parameter
	varOrder=[]
	lsList=[]
	for i,X in enumerate(symbolList):
		# save order for printing lists
		var=X.name
		varOrder.append(var)
		# collect Dict (stochastic) variable
		localDict=ParameterDict[indexDict[var]]
		xMatrix[:,i]=[localDict['m'],localDict['s']]
		# take derivative of limitstate function
		ls=sy.diff(limitstate,X)
		lsList.append(ls)

	## evaluate limit state - mean value
	limitstateEval = sy.lambdify(tuple(varOrder), limitstate, 'numpy')	
	m=LS_evaluation(limitstateEval,nvar,xMatrix); m=m[0]

	s2=0
	for i in np.arange(nvar):
		functionEval = sy.lambdify(tuple(varOrder), lsList[i], 'numpy')	
		d=LS_evaluation(functionEval,nvar,xMatrix)
		if type(d)!=int: d=d[0]
		s2=s2+d**2*xMatrix[1,i]**2

	return m,np.sqrt(s2)

def MDRMG(varDict,L=5,outdir='TMP'):
	#------------------------------------------------------------------------------------------------------------
	# MDRM-G input samples
	#
	# Input:
	# 	varDict = Dictionary with parameters stoch variables (in function of distribution) - cfr. standard format
	#	L = Gauss integration order
	# Output:
	#	pd.DataFrame of input values
	#	pd.DataFrame of sampling scheme
	# concept MaxEnt_sampling.py RVC 06.2018
	#------------------------------------------------------------------------------------------------------------

	## input assessment ##
	n = len(varDict.keys()) # number of stochastic variables - does not correct for DET variables
	if L % 2 == 0:
		print("L is even - recommended to apply odd number of integration points for improved accuracy at same cost")
	else:
		nSim=(L-1)*n+1 # number of sample points

	## Gauss points and quantiles X, considering L ##
	points=GaussPoints(L)
	r_realizations=statFunc.F_Normal(points,0,1) # quantiles corresponding with Gauss points
	r_realizations=r_realizations.flatten()

	## realizations per variable ##
	GaussPoint_df=pd.DataFrame(index=np.arange(1,L+1))
	varList=pd.Series(index=np.arange(n)+1)
	for i,var in enumerate(varDict):
		l=i+1 # stochvar number sampling scheme
		stochvar=varDict[var]
		samplepoints=ParameterRealization_r(stochvar,r_realizations)
		GaussPoint_df[l]=samplepoints
		varList[l]=stochvar['name']

	## assign in sampling scheme ##
	samplingScheme=GaussSampleScheme(L,n,nSim)
	samples_modelInput=CalculationPoints(samplingScheme,GaussPoint_df,nSim)

	## output reference
	name='MDRMGauss_samples'

	## print results
	reference=deepcopy(samples_modelInput)
	reference.columns=varList[reference.columns]

	df_writeToExcel([samples_modelInput,samplingScheme,reference],outdir+'/'+name,['modelInput','samplingScheme','modelInput_named'])

	# file reference copy - cfr. MaxEnt_sampling.py when defining stochVar from *.xlsx
	# # file stochastic variable reference path for future reference
	# text_file = open(outdir+"/REF_input_stoch.txt", "w")
	# text_file.write("Stochastic variable reference file:\n%s\n\nsheet:\n%s" %(filename,sheet)) 
	# text_file.close()

def ImportanceSampling(n_sim,ParameterList,ParameterList_IS):
    ### importance sampling functionality; LS independent (does not itself evaluate the LS)
    ### note: ParameterList and ParameterDict_IS need to be in the same order
    X=pd.DataFrame(); k=np.ones(n_sim) # initialization output and k-factor
    for i,var in enumerate(ParameterList_IS):
        X[var['name']]=MCS_var(var,n_sim) # sampling according to ParameterList_IS
        kvar=IS_factor(X[var['name']],ParameterList[i],var)
        k*=kvar
    X['IS_factor']=k
    return X

def MCS(n_sim,ParameterList):
    ### crude monte carlo functionality; LS independent (does not itself evaluate the LS)
    X=pd.DataFrame(); k=np.ones(n_sim) # initialization output and k-factor
    for i,var in enumerate(ParameterList):
        X[var['name']]=MCS_var(var,n_sim) # sampling according to ParameterList
    return X	


##################
## AUX FUNCTION ##
##################

def IS_factor(x,varDictX,varDictXis):
	# likelihood ratio within importance sampling
	return np.exp(np.log(fx(varDictX,x))-np.log(fx(varDictXis,x)))


def ParameterRealization_r(varDict,rArray):
	#------------------------------------------------------------------------------------------------------------
	# Realizations stochastic variable
	#
	# Input:
	# 	varDict = Dictionary with function parameters (in function of distribution)
	#		'name' = name of parameter: 'X'
	#		'Dist' = Distribution type: one of the following:
	#				'N' : Normal
	#				'LN' : Lognormal
	#				'MixedLN' : Mixed Lognormal (probability weighted contributions of LN constituents)
	#				'G' : Gumbel
	#				'DET' : Deterministic
	#		'm' = mean value X - or 'mi' = pd.Series of constituent mean values
	#		's' = standard deviation X - or 'si' = pd.Series of constituent standard deviations
	#		'P' = pd.Series with probabilities of constituents (cfr. MixedLN)
	#		'info' = further notes
	#	rArray = np.array (or commutable) with quantile realizations ri (0,1)
	#
	# Output:
	#	parameter realizations xi
	# rvcpy - 2017
	#------------------------------------------------------------------------------------------------------------
	DistType=varDict['dist']
	if DistType=='normal':
		return statFunc.Finv_Normal(rArray,varDict['m'],varDict['s'])
	if DistType=='lognormal':
		return statFunc.Finv_Lognormal(rArray,varDict['m'],varDict['s'])
	if DistType=='mixedlognormal':
		return statFunc.Finv_MixedLN(rArray,varDict['mi'],varDict['si'],varDict['Pi'])
	if DistType=='gumbel':
		return statFunc.Finv_Gumbel(rArray,varDict['m'],varDict['s'])
	if DistType=='deterministic':
		return np.ones(np.shape(rArray))*varDict['m']
	if DistType=='beta':
		return statFunc.Finv_Beta_ab(rArray,varDict['m'],varDict['s'],varDict['theta1'],varDict['theta2'])
	if DistType=='gamma':
		return statFunc.Finv_Gamma(rArray,varDict['m'],varDict['s'])
	if DistType=='uniform':
		return statFunc.Finv_Uniform(rArray,varDict['theta1'],varDict['theta2'])
	if DistType=='weibull':
		return statFunc.Finv_Weibull(rArray,varDict['m'],varDict['s'])
	if DistType=='lognormal_truncated':
		return statFunc.Finv_Lognormal_truncated(rArray,varDict['m'],varDict['s'],varDict['theta1'])

def LS_evaluation(LS_eval,nvar,xMatrix):

	#
	### TO BE IMPROVED ###
	# a list can be unpacked by adding *
	# e.g. LS_eval(*xMatrix)
	# unclear what this will do what a 2D array...
	#

	# currently hardcoded number of subs
	# more Pythonic code? !?!?!?
	if nvar==1:
		lsEvalList=LS_eval(xMatrix[:,0])
	elif nvar==2:
		lsEvalList=LS_eval(xMatrix[:,0],xMatrix[:,1])
	elif nvar==3:
		lsEvalList=LS_eval(xMatrix[:,0],xMatrix[:,1],xMatrix[:,2])
	elif nvar==4:
		lsEvalList=LS_eval(xMatrix[:,0],xMatrix[:,1],xMatrix[:,2],xMatrix[:,3])
	elif nvar==5:
		lsEvalList=LS_eval(xMatrix[:,0],xMatrix[:,1],xMatrix[:,2],xMatrix[:,3],xMatrix[:,4])
	elif nvar==6:
		lsEvalList=LS_eval(xMatrix[:,0],xMatrix[:,1],xMatrix[:,2],xMatrix[:,3],xMatrix[:,4],xMatrix[:,5])
	## to be continued

	return lsEvalList

def GaussPoints(n):
	#------------------------------------------------------------------------------------------------------------
	#  "Probabilistic" Gauss Hermite integration points - option to add others
	#
	# Input:
	# 	n = order of Gauss integration
	#
	# Output:
	#	array of Gauss integration points
	#
	# rvc - 2015
	#------------------------------------------------------------------------------------------------------------

	## Traditional [Physicist] Gauss Hermite integration points
	if n==2:
		points=[-0.707106781187,0.707106781187]
	if n==3:
		points=[-1.22474487139,0.,1.22474487139]
	if n==4:
		points=[-1.65068012389,-0.524647623275,0.524647623275,1.65068012389]
	if n==5:
		points=[-2.02018287046,-0.958572464614,0.,0.958572464614,2.02018287046]
	if n==6:
		points=[-2.35060497367,-1.33584907401,-0.436077411928,0.436077411928,1.33584907401,2.35060497367]
	if n==7:
		points=[-2.65196135684,-1.67355162877,-0.816287882859,0.,0.816287882859,1.67355162877,2.65196135684]
	if n==8:
		points=[-2.93063742026,-1.9816567567,-1.15719371245,-0.381186990207,0.381186990207,1.15719371245,1.9816567567,2.93063742026]
	if n==9:
		points=[-3.19099320178,-2.26658058453,-1.46855328922,-0.723551018753,0.,0.723551018753,1.46855328922,2.26658058453,3.19099320178]
	if n==10:
		points=[-3.43615911884,-2.53273167423,-1.7566836493,-1.03661082979,-0.342901327224,0.342901327224,1.03661082979,1.7566836493,2.53273167423,3.43615911884]
	if n==11:
		points=[-3.66847084656,-2.78329009978,-2.02594801583,-1.32655708449,-0.656809566882,0.,0.656809566882,1.32655708449,2.02594801583,2.78329009978,3.66847084656]
	if n==12:
		points=[-3.88972489787,-3.02063702512,-2.2795070805,-1.59768263515,-0.94778839124,-0.314240376254,0.314240376254,0.94778839124,1.59768263515,2.2795070805,3.02063702512,3.88972489787]

	if n==17:
		points=[-4.87134519367,-4.06194667588,-3.37893209114,-2.7577629157,-2.17350282667,-1.61292431422,-1.06764872574,-0.531633001343,0.,0.531633001343,1.06764872574,1.61292431422,2.17350282667,2.7577629157,3.37893209114,4.06194667588,4.87134519367]

	# multiplication sqrt(2) for "Probabilistic" Gauss-Hermite weights	
	points= [z * np.sqrt(2) for z in points] 
	PointsList=pd.DataFrame(points,index=np.arange(1,n+1,1),columns=['zj'])
	return PointsList 

def GaussSampleScheme(L,n,nSim):

    # initialize
    scheme=pd.DataFrame(index=np.arange(1,nSim+1),columns=['j','l'])    

    for l in np.arange(n+1):

        if l==0:
            scheme.loc[1,:]=[0,0] # starting entry as median value realization
        else:
            scheme.loc[2+4*(l-1):2+4*l-1,:]=[[1,l],[2,l],[4,l],[5,l]]    

    return scheme


def CalculationPoints(samplingScheme,GaussPoint_df,nSim):

    modelInput=pd.DataFrame(index=np.arange(1,nSim+1),columns=GaussPoint_df.columns)

    # initialize median values for all parameter realizations
    for var in modelInput.columns:
        modelInput[var]=GaussPoint_df.loc[3,var]

    # correct on every line the single modified Gauss point
    for i in modelInput.index:

        # j,l realization
        [j,l]=samplingScheme.loc[i,:]

        # assignment
        if l!=0: # 0-value corresponds with median point
            modelInput.loc[i,l]=GaussPoint_df.loc[j,l]

    return modelInput


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":

	### test LHS ###
	# N=10000
	# K=8
	# Z=LHS(N,K,True,True)

	# LHSpath='C:\\Users\\rvcoile\\Google Drive\\Research\\Codes\\refValues\\LHScenter_10000_8var.xlsx'
	# sheet='r'
	
	# out=pd.DataFrame(Z,index=np.arange(N),columns=np.arange(K))

	# df_writeToExcel(out,LHSpath,sheet)

	### test random realization ###
    # Concrete slab #
    cover = 0.035   # concrete cover [m]
    sig_cover = 0.010 # st.dev. of concrete cover [m]
    rebar = 0.01  #rebar diameter [m]
    dist = 0.1  #distance between rebars [m]
    w = 1.    #slab width [m] 
    As=createStochVar(dist='normal',mean=0.25*(np.pi)*rebar**2*(w/dist)*1.02,std=0.02*0.25*(np.pi)*rebar**2*(w/dist)*1.02,dim='[m2]',name='As [m2]')
    print(MCS_var(As,100))


"""Tools for creation of Abaqus material models for heat transfer analysis
Notes
-----
The thermal properties tools module supports temperature dependant material definitions for concrete, steel and prestressing steel
"""

###############
### IMPORTS ###
###############

import numpy as np
import pandas as pd
from magnelPy.SFE import ThermalTools
from scipy.interpolate import interp1d



#################################################
### Concrete irrecoverable thermal properties ###
#################################################

def c_density_EC_T_irr(T,rho=2400,units='mm'):
	""" concrete density at elevated temperature (influenced by water loss), 
	accord. EN 1992-1-2 (pp. 28)
	
	Parameters
	----------
	T : np.array
		array of Temperatures in range 20 - 1200 [°C]
	
	rho : float (optional)
		Concrete density at room temperature [kg/m3] (Default: 2400)
		
	units : String (optional)
		'mm' = outputs results in SI-mm units (default)
		'm'  = outputs results in SI units
	
	Returns
	-------
	out : pd.DataFrame
		Concrete density data in 3 columns for Abaqus input	| density | Temperature [°C] | Max. temperature [°C] |
	"""
	
	T_list = [20,115,200,400,1200]
	k_rho_list = [1,1,0.98,0.95,0.88] 
	k_rho_data = interp1d(T_list,k_rho_list)
	k_rho = k_rho_data(T)
	
	rho_c = k_rho*rho
	Tmax = np.append(T,T[1:])
	T = np.append(np.full((len(T)-1,),20),T)
	rho_c = np.append(rho_c,rho_c[1:])
	
	if units == 'm':
		out = pd.concat([pd.DataFrame(rho_c),pd.DataFrame(T),pd.DataFrame(Tmax)],axis=1)
		out.columns = ['rho [kg/m3]','T [°C]','Tmax [°C]']
	if units == 'mm':
		out = pd.concat([pd.DataFrame(rho_c*1e-12),pd.DataFrame(T),pd.DataFrame(Tmax)],axis=1)
		out.columns = ['rho [ton/mm3]','T [°C]','Tmax [°C]']
	
	return out


def c_conductivity_EC_T_irr(T,limit='lower',units='mm'):
	""" concrete thermal conductivity at elevated temperature
	accord. EN 1992-1-2 (pp. 28)
	
	Parameters
	----------
	T : np.array
		(array of) temperatures in range 20 - 1200 [°C]
	
	limit : String (optional)
		'lower' = Siliceous aggregates (default)
		'upper' = Calcareous aggregates
	
	units : String (optional)
		'mm' = outputs results in SI-mm units (default)
		'm'  = outputs results in SI units
	
	Returns
	-------
	out : pd.DataFrame
		Concrete conductivity data in 3 columns for Abaqus input	| conductivity | Temperature [°C] | Max. temperature [°C] |
	"""
	
	if limit == 'lower': lambda_c =  1.36 - 0.1360*(T/100) + 0.0057*(T/100)*(T/100)
	if limit == 'upper': lambda_c =  2.00 - 0.2451*(T/100) + 0.0107*(T/100)*(T/100)
	
	Tmax = np.append(T,T[1:])
	T = np.append(np.full((len(T)-1,),20),T)
	lambda_c = np.append(lambda_c,lambda_c[1:])
	
	out = pd.concat([pd.DataFrame(lambda_c),pd.DataFrame(T),pd.DataFrame(Tmax)],axis=1)
	if units == 'm': out.columns = ['k [W/m K]','T [°C]','Tmax [°C]']
	if units == 'mm': out.columns = ['k [mW/mm K]','T [°C]','Tmax [°C]']
		
	return out


def c_specific_heat_EC_T_irr(T, moisture=3,units='mm'):
	""" concrete specific heat at elevated temperature
	accord. EN 1992-1-2 (pp. 28)
	
	Parameters
	----------
	T : np.array
		(array of) temperatures in range 20 - 1200 [°C]
	
	moisture : float (percentage)
		moisture content, percentage of concrete weight [w_%] in range 0 - 3 
	
	units : String (optional)
		'mm' = outputs results in SI-mm units (default)
		'm'  = outputs results in SI units
	
	Returns
	-------
	out : pd.DataFrame
		Concrete conductivity data in 3 columns for Abaqus input	| conductivity | Temperature [°C] | Max. temperature [°C] |
	"""
	
	cp_data = ThermalTools.c_specific_heat_EC_T(T,moisture=moisture)
	cp_data_cooling = ThermalTools.c_specific_heat_EC_T(T,moisture=0)
	
	Tmax = np.append(T,T[1:])
	T = np.append(np.full((len(T)-1,),20),T)
	cp = np.append(cp_data_cooling,cp_data[1:])
	
	
	if units == 'm': 
		out = pd.concat([pd.DataFrame(cp),pd.DataFrame(T),pd.DataFrame(Tmax)],axis=1)
		out.columns = ['cp [J/kg K]','T [°C]','Tmax [°C]']
	if units == 'mm': 
		out = pd.concat([pd.DataFrame(cp*1e6),pd.DataFrame(T),pd.DataFrame(Tmax)],axis=1)
		out.columns = ['cp [mJ/ton K]','T [°C]','Tmax [°C]']
		
	return out

#################################################
### Steel irrecoverable thermal properties ###
#################################################

# TODO


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":
	
	# TEST 1: Irreversible concrete density
	T = np.linspace(20,500,25)
	print(c_specific_heat_EC_T_irr(T))


import numpy as np 
import sys

import magnelPy 
import matplotlib.pyplot as plt


def aic_2_param(data,dists=["N","LN","GM","GU"]):
	""" Calculates Akaike information criterion (AIC) criteria of different kind of theoretical distributions
	Fitting of the distribution is done only based on the mean value and standard distribution
	That way the number of parametars "k" is always equal to 2 and ignored and AIC is calculated only based on the loglikelihood values
	AIC is only supposed to be used to compare difrent models so this modification is ok

	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributins are fitted

	dists : list of strings
		distrbutions for which the AIC is calculated

		"N":	Normal
		"LN": 	Lognormal
		"GM": 	Gamma
		"GU": 	Gumbel
		"B": 	Beta 4,4 default
		"W": 	Weibull
		"IG": 	Inverese Gauss
		"LG": 	Logistic
		"LLG": 	LogLogistic
		"R": 	Rican
		"NAK": 	Nakagami
		"BS": 	Birnbaum Saunders

	Returns
	-------
	loglike : numpy array
		array with the AIC values
	 
	"""
	
	m=np.mean(data)
	s=np.std(data)
		
	
	loglike=np.zeros(len(dists))
	num=0
	dists=np.array(dists)
	if np.any(dists=="N"):#Normal
		loglike[num]=np.sum(np.log(statFunc.f_Normal(data,m,s)))
		
		num+=1
	if np.any(dists=="LN"):#Lognormal
		loglike[num]=np.sum(np.log(statFunc.f_Lognormal(data,m,s)))
		num+=1
	if np.any(dists=="GM"):#Gamma
		loglike[num]=np.sum(np.log(statFunc.f_Gamma(data,m,s)))
		num+=1
	if np.any(dists=="GU"):#Gumbel
		loglike[num]=np.sum(np.log(statFunc.f_Gumbel(data,m,s)))
		num+=1
	if np.any(dists=="B"): #Beta 4,4 default 
		loglike[num]=np.sum(np.log(statFunc.f_Beta_ab(data,4,4,m,s)))
		num+=1
	if np.any(dists=="W"): #Weibull
		loglike[num]=np.sum(np.log(statFunc.f_Weibull(data,m,s)))
		num+=1
	if np.any(dists=="IG"): #Inverese Gauss
		loglike[num]=np.sum(np.log(statFunc.f_InvGauss(data,m,s)))
		num+=1
	if np.any(dists=="LG"): #Logistic
		loglike[num]=np.sum(np.log(statFunc.f_Logistic(data,m,s)))
		num+=1
	if np.any(dists=="LLG"): #LogLogistic
		loglike[num]=np.sum(np.log(statFunc.f_LogLogistic(data,m,s)))
		num+=1
	if np.any(dists=="R"): #Rican
		loglike[num]=np.sum(np.log(statFunc.f_Rican(data,m,s)))
		num+=1
	if np.any(dists=="NAK"): #Nakagami
		loglike[num]=np.sum(np.log(statFunc.f_Nakagami(data,m,s)))
		num+=1
	if np.any(dists=="BS"): #Birnbaum Saunders
		loglike[num]=np.sum(np.log(statFunc.f_Birnbaum_Saunders(data,m,s)))
		num+=1
		
	return -loglike

def AIC_Norm(data):
	""" Calculates Akaike information criterion (AIC) criteria for normal distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		loc,scale=stats.norm.fit(data)
		AIC=stats.norm.logpdf(data,loc,scale).sum()+2*2
	
	except:
		AIC,loc,scale=[float('NaN'),float('NaN'),float('NaN')]
	
	return AIC,loc,scale

def AIC_Lognormal(data):
	""" Calculates Akaike information criterion (AIC) criteria for lognormal distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	s,loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		s,loc,scale=stats.lognorm.fit(data)
		AIC=stats.lognorm.logpdf(data,s,loc,scale).sum()+2*3
	except:
		AIC,s,loc,scale=[float('NaN'),float('NaN'),float('NaN'),float('NaN')]
	
	return AIC,s,loc,scale

def AIC_Gamma(data):
	""" Calculates Akaike information criterion (AIC) criteria for gamma distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	a,loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		a,loc,scale=stats.gamma.fit(data,floc=0)
		print(loc)
		AIC=stats.gamma.logpdf(data,a,loc,scale).sum()+2*3
	except:
		AIC,a,loc,scale=[float('NaN'),float('NaN'),float('NaN'),float('NaN')]
		
	return AIC,a,loc,scale

def AIC_Nakagami(data):
	""" Calculates Akaike information criterion (AIC) criteria for Nakagami distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	a,loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		a,loc,scale=stats.nakagami.fit(data)
		AIC=stats.nakagami.logpdf(data,a,loc,scale).sum()+2*3
	except:
		AIC,a,loc,scale=[float('NaN'),float('NaN'),float('NaN'),float('NaN')]
	
	return AIC,a,loc,scale

def AIC_Gumbel(data):
	""" Calculates Akaike information criterion (AIC) criteria for Gumbel distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		loc,scale=stats.gumbel_r.fit(data)
		AIC=stats.gumbel_r.logpdf(data,loc,scale).sum()+2*2
	except:
		AIC,loc,scale=[float('NaN'),float('NaN'),float('NaN')]
	
	return AIC,loc,scale

def AIC_Beta(data,alpha=4,beta=4):
	""" Calculates Akaike information criterion (AIC) criteria for beta distributions with fixed values of alpha and beta
		
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	a,b,loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		a,b,loc,scale=stats.beta.fit(data,fix_a=alpha,fix_b=beta)
		AIC=stats.beta.logpdf(data,a,b,loc=loc,scale=scale).sum()+2*2
	
	except:
		AIC,a,b,loc,scale=[float('NaN'),float('NaN'),float('NaN'),float('NaN'),float('NaN')]

	return AIC,a,b,loc,scale

def AIC_Weibull(data):
	""" Calculates Akaike information criterion (AIC) criteria for Weibull distributions
	
	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributin is fitted
	
	Returns
	-------
	AIC : float
		AIC value
	c,loc,scale : floats
		distribution parameters

	Note
	-------
	In case of the error it returns all "NaN"
	 
	"""
	try:
		c,loc,scale=stats.weibull_min.fit(data)
		AIC=stats.weibull_min.logpdf(data,c,loc,scale).sum()+2*3
	
	except:
		AIC,c,loc,scale=[float('NaN'),float('NaN'),float('NaN'),float('NaN')]
		
	return AIC,c,loc,scale

def aic_full(data,dists=["N","LN","GM","GU"]):
	""" Calculates Akaike information criterion (AIC) criteria of different kind of theroetical distributions
	

	Parameters
	----------	
	data : 1D numpy array or list
		data for which the distributins are fitted

	dists : list of strings
		distrbutions for which the AIC is calculated

		"N":	Normal
		"LN": 	Lognormal
		"GM": 	Gamma
		"GU": 	Gumbel
		"B": 	Beta 4,4 default
		"W": 	Weibull
		"NAK": 	Nakagami
		
	Returns
	-------
	loglike : numpy array
		array with the AIC values
	 
	"""
		
	loglike=np.zeros(len(dists))
	num=0
	dists=np.array(dists)
	if np.any(dists=="N"):#Normal
		loglike[num],_,_=AIC_Norm(data)
		num+=1
	if np.any(dists=="LN"):#Lognormal
		loglike[num],_,_,_=AIC_Lognormal(data)
		num+=1
	if np.any(dists=="GM"):#Gamma
		loglike[num],_,_,_=AIC_Gamma(data)
		num+=1
	if np.any(dists=="GU"):#Gumbel
		loglike[num],_,_=AIC_Gumbel(data)
		num+=1
	if np.any(dists=="B"): #Beta 4,4 default 
		loglike[num],_,_,_,_=AIC_Beta(data)
		num+=1
	if np.any(dists=="W"): #Weibull
		loglike[num],_,_,_=AIC_Weibull(data)
		num+=1
	if np.any(dists=="NAK"): #Nakagami
		loglike[num],_,_,_=AIC_Nakagami(data)
		num+=1
		
	return -loglike

if __name__ == "__main__":
	
	# print('Test')
	# Pn_input =np.load("Pn8.npy")
	# names=[r'$E_{Confab}$',r'$E_{H&S}$',r'$E_{Iqbal}$',r'$E_{Confab}$',r'$E_{H&S}$',r'$E_{Iqbal}$',r'$E_{Confab}$',r'$E_{H&S}$']
	# #ploty_pdf(Pn_input[:,:3],names)
	# ploty_cdf(Pn_input[:,:6],names,BW=True,figname='blabla')
	# plt.show()
	test_data=np.random.normal(3,0.5,10*10)
	plt.figure()
	AIC=aic_2_param(test_data,dists=["N","LN","GM","GU"])
	print(AIC)
	plt.bar(np.arange(len(AIC)),AIC,tick_label=["N","LN","GM","GU"])

	# plt.figure()
	# AIC2=aic_full(test_data,dists=["N","LN","GM","GU"])
	# print(AIC2)
	# plt.bar(np.arange(len(AIC2)),AIC2,tick_label=["N","LN","GM","GU"])
	plt.show()


"""
magnelPy
========
Provides
  1. Support methods for data handling
  2. Probabilistic / Reliability methods and tools
  3. Structural Fire Engineering methods and tools

Available subpackages
---------------------
SFE
  Structural Fire Engineering codes and tools
SAFIRshell
  SAFIR support codes
ML
  advanced regression support codes

Available Utilities
---------------------
admin
	Basic functions for data handling : read-write to excel etc.
stochVar
	Standard formatting of stochastic variables.
statFunc
	Probabilistic distribution evaluations in standardized format.
sampling
	Sampling procedures.

Documentation
-------------
The documentation is incomplete and under development.

Code snippets are indicated by three greater-than signs:
  >>> x = 42
  >>> x = x + 1

Use the built-in ``help`` function to view a function's docstring:
  >>> help(magnelPy)

"""

### standard imports ###
# makes e.g. magnelPy.statFunc directly available when >>> import magnelPy
from magnelPy import statFunc
from magnelPy import admin



"""MaterialProperties\\UserDefined
Notes
-----
Temperature-dependent properties for use in SFE.
"""

#############
## IMPORTS ##
#############

import numpy as np
import pandas as pd

############
## MODULE ##
############

class UserDefined(object):
	
	def __init__(self,temperature,conductivity,specific_heat,density):
		""" Initialization of User Defined Material

		Parameters
		----------	
		temperature : np.array (n,)
			Temperatures for which data is provided [°C]
		conductivity : np.array (n,)
			Conductivity for given temperature [W/(m*K)]
		specific_heat : np.array (n,)
			Specific heat for given temperature [J/(kg*K)]
		density : np.array (n,)
			Density for given temperature [kg/m3]

		Returns
		-------
		-
		
		Example use
		-----------
		import magnelPy.SFE.MaterialProperties as mat
		from magnelPy.SFE.MaterialProperties import SFRM as sfrm
		import numpy as np
		T=np.array([20,100,200,300,400,500,600])
		k=sfrm.ki(T)
		c=sfrm.cp(T)
		r=sfrm.rho(T)
		ins=mat.UserDefined(T,k,c,r)
		print(ins.data)
		>>>         k            c           r
			20   0.068407   821.710734  284.577910
			100  0.079424   924.583736  262.721108
			200  0.095388  1031.123125  253.898015
			300  0.114115  1117.924596  248.892572
			400  0.135986  1188.984460  245.408915
			500  0.161419  1247.403953  242.744797
			600  0.190863  1295.601815  240.592363
		"""
		aux=pd.DataFrame([conductivity,specific_heat,density],columns=temperature,index=['k','c','r'])
		self.data=aux.transpose()
		
	def rho(self,theta):
		""" Temperature-dependent density

		Parameters
		----------	
		theta : float or np.array (n,)
			Temperatures for which data is requested

		Returns
		-------
		Interpolated density [kg/m3]
		"""
		return np.interp(theta,self.data.index,self.data['r'].values,left=np.NAN,right=np.NAN)

	def cp(self,theta):
		""" Temperature-dependent specific heat

		Parameters
		----------	
		theta : float or np.array (n,)
			Temperatures for which data is requested

		Returns
		-------
		Interpolated specific heat [J/(kg*K)]
		"""
		return np.interp(theta,self.data.index,self.data['c'].values,left=np.NAN,right=np.NAN)
	
	def k(self,theta):
		""" Temperature-dependent conductivity

		Parameters
		----------	
		theta : float or np.array (n,)
			Temperatures for which data is requested

		Returns
		-------
		Interpolated conductivity [W/(m*K)]
		"""
		return np.interp(theta,self.data.index,self.data['k'].values,left=np.NAN,right=np.NAN)








"""
magnelPy.SFE
============
Provides
  1. Fire definition
  2. Thermal calculation tools
  3. Structural calculation tools
  4. Material properties

=============== ==========================================================
Fire definition ~ FireCurve
==========================================================================
ISO834          ISO834 standard fire definition
=============== ==========================================================

"""

### get submodules ###
from magnelPy.SFE import FireCurve
from magnelPy.SFE import ThermalTools
from magnelPy.SFE import Radiation
from magnelPy.SFE import MaterialProperties



### get command line functionality ###
from magnelPy.SFE.ThermalTools import EC_concreteSlab_ISO834
from magnelPy.SFE.ThermalTools import EC_concreteSlab_ECparametric




"""Test file for irrecoverable Abaqus material models
"""

###############
### IMPORTS ###
###############

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from magnelPy.Abaqus import MechanicalProperties
from magnelPy.Abaqus import ThermalProperties
from scipy.interpolate import interp1d

SW_test = 3


def track_Tmax(T_list):
	max_T = -1
	T_max_list = []
	
	for T in T_list:
		if T > max_T: max_T = T
		T_max_list.append(max_T)
	
	return(T_max_list)


def abaqus_interpolation(T_list,Tmax_list,df):
	
	T_rise_data = df.loc[df['T [°C]'] == df['Tmax [°C]']]
	T_fall_data = pd.concat([df.iloc[[0]],df.loc[df['T [°C]'] != df['Tmax [°C]']]])
	out = []
	
	print(T_fall_data)
	
	for T,Tmax in zip(T_list,Tmax_list):
		if T >= Tmax:
			out_data = interp1d(T_rise_data.iloc[:, 1],T_rise_data.iloc[:, 0])
			out.append(float(out_data(T)))
		if T < Tmax: 
			out_data = interp1d(T_fall_data.iloc[:, 2],T_fall_data.iloc[:, 0])
			out.append(float(out_data(Tmax)))
			
	return(out)


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":
	
	# TEST 1: Concrete irrecoverable density
	if SW_test == 1:
		rho_data = ThermalProperties.c_density_EC_T_irr(np.arange(20,801,20),units='m')
		
		# simulate heating regime: 20°C -> 500°C -> 20°C
		T = np.concatenate((np.arange(20,501,20),np.arange(500,19,-20),np.arange(20,701,20)))
		Tmax = track_Tmax(T)
		dens = abaqus_interpolation(T,Tmax,rho_data)
		
		fig, ax1 = plt.subplots()
		ax2 = ax1.twinx()
		ax1.plot(dens,'r')
		ax2.plot(T)
		ax2.plot(Tmax,'m--')
		
		ax1.set_ylabel('Density')
		ax2.set_ylabel('Temperature [°C]')
		ax1.set_xlabel('Time')
	
	
	# TEST 2: Concrete irrecoverable conductivity
	if SW_test == 2:
		rho_data = ThermalProperties.c_conductivity_EC_T_irr(np.arange(20,801,20))
		
		# simulate heating regime: 20°C -> 500°C -> 20°C
		T = np.concatenate((np.arange(20,501,20),np.arange(500,19,-20),np.arange(20,701,20)))
		Tmax = track_Tmax(T)
		k = abaqus_interpolation(T,Tmax,rho_data)
		
		fig, ax1 = plt.subplots()
		ax2 = ax1.twinx()
		ax1.plot(k,'r')
		ax2.plot(T)
		ax2.plot(Tmax,'m--')
		
		ax1.set_ylabel('Conductivity')
		ax2.set_ylabel('Temperature [°C]')
		ax1.set_xlabel('Time')
	
	
	# TEST 3: Concrete irrecoverable specific heat
	if SW_test == 3:
		k_data = ThermalProperties.c_specific_heat_EC_T_irr(np.arange(20,801,5),moisture=1,units='m')
		
		# simulate heating regime: 20°C -> 500°C -> 20°C
		T = np.concatenate((np.arange(20,101,5),np.arange(110,19,-5),np.arange(20,501,5),np.arange(500,19,-5)))
		Tmax = track_Tmax(T)
		cp = abaqus_interpolation(T,Tmax,k_data)
		
		fig, ax1 = plt.subplots()
		ax2 = ax1.twinx()
		ax1.plot(cp,'r',label='specific heat')
		ax2.plot(T,label='Temperature')
		ax2.plot(Tmax,'m--',label='Max temperature')
		
		ax1.set_ylabel('Conductivity')
		ax2.set_ylabel('Temperature [°C]')
		ax1.set_xlabel('Time')
		ax1.set_ylim(800,2400)
		ax2.set_ylim(0,800)
		
		ax1.legend()
		ax2.legend()
	
	# TEST 4: Concrete irrecoverable thermal expansion
	if SW_test == 4:
		a_data = MechanicalProperties.c_thermal_expansion_coef_EC_irr(delta=10)
		
		# simulate heating regime: 20°C -> 500°C -> 20°C
		T = np.concatenate((np.arange(20,401,5),np.arange(400,19,-5)))
		Tmax = track_Tmax(T)
		a = abaqus_interpolation(T,Tmax,a_data)
		
		fig, ax1 = plt.subplots()
		ax2 = ax1.twinx()
		ax1.plot(a,'r',label='specific heat')
		ax2.plot(T,label='Temperature')
		ax2.plot(Tmax,'m--',label='Max temperature')
		
		ax1.set_ylabel('Conductivity')
		ax2.set_ylabel('Temperature [°C]')
		ax1.set_xlabel('Time')
		ax1.set_ylim(800,2400)
		ax2.set_ylim(0,800)
		
		ax1.legend()
		ax2.legend() 
	
	print('done')
		


"""MaterialProperties\\Steel
Notes
-----
Temperature-dependent properties for use in SFE.
"""

#############
## IMPORTS ##
#############

import numpy as np

############
## MODULE ##
############

def ky_EC3(theta):
	""" Steel yield stress retention factor according to EN 1993-1-2:2005

	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	Returns
	-------
	ky_EC3 : float or np.array
		 retentionfactor for steel yield stress [-]
		 EN 1993-1-2:2005, Table 3.1
	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import Steel as steel
	import numpy as np	
	theta=np.arange(100,1200,100)
	kfy=steel.ky_EC3(theta)
	print(kfy)
	>>> [1.   1.   1.   1.   0.78 0.47 0.23 0.11 0.06 0.04 0.02]
	"""
	T=[20,400,500,600,700,800,900,1100,1200]
	k=[1,1,0.78,0.47,0.23,0.11,0.06,0.02,10**-10]
	ky_EC3=np.interp(theta,T,k)
	return ky_EC3


def kE_EC3(theta):
	""" Steel modulus of elasticity retention factor according to EN 1993-1-2:2005

	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	Returns
	-------
	kE_EC3 : float or np.array
		 retentionfactor for steel modulus of elasticity [-]
		 EN 1993-1-2:2005, Table 3.1
	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import Steel as steel
	import numpy as np	
	theta=np.arange(100,1200,100)
	kfy=steel.kE_EC3(theta)
	print(kE)
	>>> [1.     0.9    0.8    0.7    0.6    0.31   0.13   0.09   0.0675 0.045 0.0225]
	"""
	T=[20,100,500,600,700,800,1100,1200]
	k=[1,1,0.6,0.31,0.13,0.09,0.0225,10**-10]
	kE_EC3=np.interp(theta,T,k)
	return kE_EC3

def density_EC3(theta=None):
	""" Steel density according to EN 1993-1-2:2005

	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	Returns
	-------
	rho_EC3 : float
		 density [kg/m3]
		 EN 1993-1-2:2005, 3.2.2
	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import Steel as steel
	print(steel.density_EC3())
	>>> 7850.0
	"""
	if  isinstance(theta,np.ndarray):
		rho_EC3 = 7850.0*np.ones(np.shape(theta)) # [kg/m3]
	else:
		rho_EC3 = 7850.0 # [kg/m3]
	return rho_EC3

def cp_EC3(theta):
	""" Steel specific heat according to EN 1993-1-2:2005;
	note: constant specific heat maintained above 1200 degrees C for numerical purposes

	Parameters
	----------	
	theta : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]

	Returns
	-------
	cp : float or np.array
		 specific heat [J/kgK]
		 EN 1993-1-2:2005, 3.4.1.2
	
	Example use
	-----------
	from magnelPy.SFE.MaterialProperties import Steel as steel
	import numpy as np	
	theta=np.arange(100,500,100)
	cp=steel.cp_EC3(theta)
	print(cp)
	>>> [487.62 529.76 564.74 605.88]
	"""

	cp=np.select([theta<600, theta<735, theta<900, theta<=1200, theta<2000],  
            [425+0.773*theta-1.69/1000*theta**2+2.22*10**-6*theta**3, 666+13002/(738-theta), 545+17820/(theta-731),650,650])

	return cp





"""Mechanical Calculation Tools
Notes
-----
The mechanical calculation tools module supports various temperature related methods for SFE.
"""

#############
## IMPORTS ##
#############

import numpy as np
from scipy.interpolate import interp1d


#################################################################
## Concrete strength and deformatoin properties in EN 1992-1-2 ##
#################################################################

def c_strength_retention_EC(fck,Temp,aggregates='sil'):
	""" obtain tabulated parameters for temperature dependant stress-strain relationships
	accord. EN 1992-1-2 Table 3.1 for NSC (pp. 20) and Table 6.1N for HSC (pp. 59)
	
	Parameters
	----------
	fck : float
		concrete characteristic compressive strength [MPa]
	
	Temp : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	
	aggregates : String
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	Returns
	-------
	k_fc : float or np.array
		Strength retention factor for concrete compressive strength [-]
	
	Ec_1 : float or np.array
		net strain at peak compressive stress [-]
	
	Ec_u : float or np.array
		ultimate net strain [-]
	
	Example use
	-----------
	>>> import numpy as np
	>>> import magnelPy.SFE as sfe
	>>> fck = 50
	>>> Temp = 500
	>>> k_fc,e_c1,e_cu = sfe.MechanicalTools.c_strength_retention_EC(fck,Temp,aggregates='calc')
	"""
	
	# Eurocode 2 data
	T_list = np.insert(np.arange(300,1300,100),0,[20,50,100,200,250])
	if fck < 55: # EN 1992-1-2 Table 3.1
		k_fc_list = [1,1,1,0.95,0.9,0.85,0.75,0.6,0.45,0.3,0.15,0.08,0.04,0.01,0] if aggregates == 'sil' else [1,1,1,0.97,0.94,0.91,0.85,0.74,0.6,0.43,0.27,0.15,0.06,0.02,0] 
	else:
		# EN 1992-1-2 Table 6.1N describes strength reduction factors for HSC (C55/67 and up) in 3 different classes
		if fck < 70: # class 1 (recommended for C55/67 and C60/75)
			k_fc_list = [1,1,0.9,0.9,0.9,0.85,0.75,0.6,0.45,0.3,0.15,0.08,0.04,0.01,0] 
		elif fck < 90: #class 2 (recommended for C70/85 and C80/95)
			k_fc_list = [1,1,0.75,0.75,0.75,0.75,0.75,0.6,0.45,0.3,0.15,0.1125,0.075,0.0375,0]
		else: # class 3 (recommended for C90/105)
			[1,1,0.75,0.7,0.675,0.65,0.45,0.3,0.25,0.2,0.15,0.08,0.04,0.01,0]
		
	e_c1_list = [0.0025,0.0030625,0.004,0.0055,0.00625,0.007,0.01,0.015,0.025,0.025,0.025,0.025,0.025,0.025,0.025] # EN 1992-1-2 Table 3.1, identical values for silicious and calcereous aggregates
	e_cu_list = np.insert(np.arange(0.0275,0.0501,0.0025),0,[0.02,0.0209375,0.0225,0.025,0.02625]) # EN 1992-1-2 Table 3.1, identical values for silicious and calcereous aggregates
	
	k_fc_data = interp1d(T_list,k_fc_list)
	e_c1_data = interp1d(T_list,e_c1_list)
	e_cu_data = interp1d(T_list,e_cu_list)
	
	k_fc = k_fc_data(Temp).round(10) # using .round(10) resolves issue with trailing digits
	e_c1 = e_c1_data(Temp).round(10)
	e_cu = e_cu_data(Temp).round(10) 
	
	return(k_fc,e_c1,e_cu)


def c_strength_retention_EC_tensile(T):
	""" concrete tensile strength reduction, accord. EN 1992-1-2 Figure 3.2 (pp. 22)
	
	Parameters
	----------	
	T : float or np.array
		(array of) Temperature in range 20 - 600 [°C] (extended beyond 600°C for numerical purposes)
	
	Returns
	-------
	k_fct : float or np.array
		Strength retention factor for concrete tensile strength [-]
	
	Example use
	-----------
	>>> TODO
	"""
	T = np.array(T).astype(float) # conversion to float, to counteract np.piecewise() for int inputs
	conds = [T < 100,(T>=100)&(T<600),T>=600]
	funcs = [1,lambda T:(600-T)/500,0]
	return np.piecewise(T, conds, funcs)


def c_thermal_strain_EC(temp,aggregates='sil'):
	""" returns the total thermal eleongation (thermal strain), with reference to the length at 20°C, accord. EN 1992-1-2 (pp. 26)
	
	Parameters
	----------	
	Temp : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	
	aggregates : String
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	Returns
	-------
	Eps : float or np.array
		total thermal elongation [-]
		
	Example use
	-----------
	>>> import numpy as np
	>>> import magnelPy.SFE as sfe
	>>> Temp = np.arange(20,1101,20)
	>>> eps = sfe.ThermalTools.c_thermal_strain_EC(Temp,aggregates='calc')	
	"""
	
	out = -1.8e-4 + 9e-6*temp + 2.3e-11*temp**3 if aggregates=='sil' else -1.2e-4 + 6e-6*temp + 1.4e-11*temp**3
	out = np.clip(out,0,0.014) if aggregates=='sil' else np.clip(out,0,0.012) # clip off when max value is reached, in accordance with EN 1992-1-2
	return out


##############################################################
## Steel strength and deformatoin properties in EN 1992-1-2 ##
##############################################################

def s_strength_retention_EC(T,steeltype='hotrolled'):
	""" returns deterministic value of the strenght retention factor for steel properties at elevated temperatures
	according to EN 1992-1-2 (pp. 23)
	
	Parameters
	----------	
	T : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	
	Steeltype : String
		'hotrolled' (default)
		'coldworked'
	
	Returns
	-------
	k_fy : float or np.array
		Strength retention factor for steel maximum stress level [-]
		
	k_sp : float or np.array
		Strength retention factor for steel proportional limit [-]
		
	k_Es : float or np.array
		Strength retention factor for steel slope of the linear elastic range [-]
	"""
	T_list = np.array([20,100,200,300,400,500,600,700,800,900,1000,1100,1200])
	
	if steeltype == 'hotrolled':
		# EN 1992-1-2 - Table 3.2a (2004), note: identical to EN 1993-1-2:2005 - Table 3.1
		kfy_list = np.array([1,1,1,1,1,0.78,0.47,0.23,0.11,0.06,0.04,0.02,0.0])
		ksp_list = np.array([1,1,0.81,0.61,0.42,0.36,0.18,0.07,0.05,0.04,0.02,0.01,0.0]) # note: fs_p = fs_y up to 100°C -> linear ascending branch in stress-stain diagram
		kE_list = np.array([1,1,0.9,0.8,0.7,0.6,0.31,0.13,0.09,0.07,0.04,0.02,0.0])
	if steeltype == 'coldworked':
		# EN 1992-1-2 - Table 3.2a (2004)
		kfy_list = np.array([1,1,1,1,0.94,0.67,0.4,0.12,0.11,0.08,0.05,0.03,0.0])
		ksp_list = np.array([1,0.96,0.92,0.81,0.63,0.44,0.26,0.08,0.06,0.05,0.03,0.02,0.0])
		kE_list = np.array([1,1,0.87,0.72,0.56,0.4,0.24,0.08,0.06,0.05,0.03,0.02,0.0])
	kfy_interpol = interp1d(T_list,kfy_list)
	ksp_interpol = interp1d(T_list,ksp_list)
	kEs_interpol = interp1d(T_list,kE_list)
	
	return kfy_interpol(T),ksp_interpol(T), kEs_interpol(T)


def s_thermal_strain_EC(temp):
	""" returns the total thermal elongation (thermal strain), with reference to the length at 20°C
	accord. EN 1992-1-2 (pp. 28-29)
	
	Parameters
	----------	
	temp : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	
	Returns
	-------
	Eps : float or np.array
		total thermal elongation of steel [-]
	"""
	out = []
	for T in temp:
		if T <= 750: out.append(-2.416e-4 + T*1.2e-5 + T*T*0.4e-8)
		else:
			if T <= 860: out.append(11e-3)
			else: out.append(-6.2e-3 + T*2e-5)
	
	return out


##########################################################################
## Prestressing teel strength and deformatoin properties in EN 1992-1-2 ##
##########################################################################

def p_thermal_strain_EC(temp):
	""" returns the total thermal elongation (thermal strain), with reference to the length at 20°C
	accord. EN 1992-1-2 (pp. 28-29)
	
	Parameters
	----------	
	temp : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
	
	Returns
	-------
	Eps : float or np.array
		total thermal elongation of prestressing steel [-]
	"""
	
	return -2.016e-4 + temp*1e-5 + temp*temp*0.4e-8


def p_strength_retention_EC(T, SteelClass='B'):
	""" returns deterministic value of the strenght retention factor for prestressing steel properties at elevated temperatures
	according to EN 1992-1-2 (pp. 25)
	
	Parameters
	----------	
	T : float or np.array
		(array of) Temperature in range 20 - 1200 [°C]
		
	SteelClass : String
		accord. EN 1992-1-1 Table C.1 (pp. 205)
		'B' 5.0% Characteristic strain at maximum force (Default) 
		'A' 2.5% Characteristic strain at maximum force
	
	Returns
	-------
	k_fy : float or np.array
		Strength retention factor for prestressing steel maximum stress level [-]
		
	k_sp : float or np.array
		Strength retention factor for prestressing steel proportional limit [-]
		
	k_Ep : float or np.array
		Strength retention factor for prestressing steel slope of the linear elastic range [-]
	"""
	
	T_list = np.array([20,100,200,300,400,500,600,700,800,900])
	
	if SteelClass == 'B':
		# EN 1992-1-2 - Table 3.3 (2004)
		kfy_list = np.array([1,0.99,0.87,0.72,0.46,0.22,0.10,0.08,0.05,0.03])
	
	if SteelClass == 'A':
		# EN 1992-1-2 - Table 3.3 (2004)
		kfy_list = np.array([1,1,0.87,0.70,0.50,0.30,0.14,0.06,0.04,0.02])
		
	ksp_list = np.array([1,0.68,0.51,0.32,0.13,0.07,0.05,0.03,0.02,0.01]) # note: fs_p = fs_y at 20°C -> linear ascending branch in stress-stain diagram
	kEp_list = np.array([1,0.98,0.95,0.88,0.81,0.54,0.41,0.10,0.07,0.03])
	
	kfy_interpol = interp1d(T_list,kfy_list)
	ksp_interpol = interp1d(T_list,ksp_list)
	kEp_interpol = interp1d(T_list,kEp_list)
	
	return kfy_interpol(T),ksp_interpol(T), kEp_interpol(T)


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":

	print("testing")


"""
magnelPy.Abaqus
============
Provides
  1. Input Data for concrete material models in a Abaqus Heat Transfer Analysis
  2. Input Data for concrete and steel material models in a Abaqus Heat Thermal Stress Analysis

"""
### get submodules ###
from magnelPy.SFE import ThermalTools, MechanicalTools
from magnelPy.Abaqus import ThermalProperties, MechanicalProperties
from magnelPy.Abaqus import AbaqusPropertiesMaster


"""Tools for creation of Abaqus material models for thermal-stress analysis
Notes
-----
The mechanical properties tools module supports temperature dependant material definitions for concrete, steel and prestressing steel
"""

###############
### IMPORTS ###
###############

import numpy as np
import pandas as pd
from scipy.interpolate import interp1d
from magnelPy.admin import df_readFromExcel
from magnelPy.SFE import MechanicalTools


####################################################
### Concrete strength and deformation properties ###
####################################################

def c_stress_strain_compression_T(fc_k,aggregates='sil',return_T_list=False):
	""" obtain datapoints for concrete stress-strain relationship at elevated temperatures, for direct input in Abaqus, in range 20 - 1100 °C
	accord. EN 1992-1-2 (pp. 20)
	
	Parameters
	----------
	fc_k : float
		concrete characteristic compressive strength (at 20°C) [MPa]
	
	aggregates : String (optional)
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	return_T_list : boolean (optional)
		self explanatory
	
	Returns
	-------
	s_out : pd.DataFrame
		Stress-(plastic) strain data in 3 columns for Abaqus input	| stress [MPa] | pl. strain [-] | Temperature [°C] |
	
	e_out : pd.DataFrame
		Elasticity data in 3 columns for Abaqus input	| Young's mod [MPa] | Poisson rat. [-] | Temperature [°C] |
		
	Example use
	-----------
	>>> TODO
	"""
	
	T_list = [20,50,75,100,200,300,400,500,600,700,800,900,1000,1100] # np.arange(20,1001,100)
	k_list,e_c1_list,e_cu_list = MechanicalTools.c_strength_retention_EC(fc_k,T_list,aggregates)
	
	sig_data=[]; ep_data=[]; E_list=[]
	
	# generate data points for each temperature in T_list
	for T,k,e_c1,e_cu in zip(T_list,k_list,e_c1_list,e_cu_list):
		fc_k_T = fc_k*k
		e_list = np.concatenate((np.array([0,0.5,0.6,0.7,0.8,0.9,0.95])*e_c1,np.array([0,0.02,0.04,0.06,0.08,0.1,0.15,0.2,0.3,0.4,0.6,0.8,1])*(e_cu - e_c1) + e_c1))
		s_list = (3*e_list*fc_k_T)/(e_c1*(2+((e_list/e_c1)**3))) # see figure 3.1, pp. 21
		E = s_list[1]/e_list[1] # tangent modulus [MPa], needed to express plastic strain in abaqus
		E_list.append(E)
		e_el_list = s_list/E # elastic strain
		e_pl_list = e_list - e_el_list # plastic strain
		e_pl_list[1] = 0 # correct cases where e_pl is very close to 0
		
		sig_data.append(np.delete(s_list,0)) # removing first element in list using np.delete() since first line in Abaqus table cannot be [0,0]
		ep_data.append(np.delete(e_pl_list,0))
	
	# repackage stress strain data for output
	sig_out = np.array(sig_data).flatten() # [MPa]
	ep_out = np.array(ep_data).flatten() # [-]
	T_out = np.repeat(T_list,len(e_list)-1) # [°C]
	
	s_out = pd.concat([pd.DataFrame(sig_out),pd.DataFrame(ep_out),pd.DataFrame(T_out)],axis=1)
	s_out.columns = ['s [MPa]','e_pl [-]','T [°C]']
	
	# repackage elasticity data for output
	E_out = np.array(E_list)
	pois_out = np.ones(len(T_list))*0.2
	
	e_out = pd.concat([pd.DataFrame(E_out),pd.DataFrame(pois_out),pd.DataFrame(T_list)],axis=1)
	e_out.columns = ['E [MPa]','v [-]','T [°C]']
	
	if return_T_list:
		return s_out, e_out, T_list
	else:
		return s_out, e_out


def c_stress_displacement_tension_T(fc, fct = -1,return_T_list=False):
	""" obtain datapoints for concrete stress-displacement relationship at elevated temperatures, for direct input in Abaqus, in range 20 - 1100 °C
	accord. EN 1992-1-2 (pp. 22) and fib MC 2010 (pp. 130)
	GF constant with increasing temperature
	
	Parameters
	----------
	fc : float
		concrete compressive strength [MPa]
	fct : float (optional)
		concrete tensile strength [MPa]
		when fct = -1, fct is calculated accoring to the fib model code 2010
	
	Returns
	-------
	out : pd.DataFrame
		Dataframe with 3 columns for Abaqus input	| stress [MPa] | displ. [mm] | Temperature [°C] | 
	"""
	if fct == -1:
		fct = c_tensile_strength_fib(fc) # [MPa]
	
	GF = c_fracture_energy_fib(fc)*0.001 # [N/m] -> [N/mm]
	
	T_list = [20,100,200,300,400,500,600,700,800,900,1000,1100]
	k_list = MechanicalTools.c_strength_retention_EC_tensile(T_list)
	
	s_data=[]; dist_data=[]
	
	# generate data points for each temperature in T_list
	for T,k in zip(T_list,k_list):
		fct_k_T = fct*k if k>0 else fct*0.1
		s_list = np.array([1,0.2,0.05])*fct_k_T # value 0.01 fct_k is used since Abaqus only allows non-zero tensile strenghts
		
		# post-peak crack opening at stress = 0.2*fctm and stress = 0
		w1 = GF/fct_k_T # [N/mm]/[N/mm²] = [mm]
		wc = 5*w1 # crack opening [mm]
		w_list = [0,w1,wc]
		
		s_data.append(s_list)
		dist_data.append(w_list)
	
	# repackage data for output
	s_out = np.array(s_data).flatten()
	w_out = np.array(dist_data).flatten()
	T_out = np.repeat(T_list,3)
	
	out = pd.concat([pd.DataFrame(s_out),pd.DataFrame(w_out),pd.DataFrame(T_out)],axis=1)
	out.columns = ['s [MPa]','w [mm]','T [°C]']
	
	out = regularize_fct(out,T_list)
	
	if return_T_list:
		return out, T_list
	else:
		return out


def c_thermal_expansion_coef_EC(aggregates='sil',delta = 20):
	""" returns a table coefcient of thermal expansion (CET) for concrete, with reference to the length at 20°C, for direct input in Abaqus
	accord. EN 1992-1-2 (pp. 26)
	Note: in abaqus thermal strains are defined as total expansion from a reference temperature
	See abaqus manual: http://dsk.ippt.pan.pl/docs/abaqus/v6.13/books/usb/default.htm?startat=pt05ch26s01abm52.html
	
	Parameters
	----------	
	aggregates : String
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	delta : float
		interval between temperature points
	
	Returns
	-------
	out : pd.DataFrame
		Dataframe with 2 columns for Abaqus input	 | CET [-] | Temperature [°C] | 
	"""
	
	if delta > 20: delta = 20
	
	T_list = np.arange(20,1101,delta)
	e_th = MechanicalTools.c_thermal_strain_EC(T_list)
	e_th[0]=0 # set first value equal to 0, EC2 analytical expression is exactly non-zero at 20°C
	
	cet = np.zeros(len(T_list))
	for i in range(0,len(T_list)):
		if i>0:
			cet[i]=(e_th[i]-e_th[0])/(T_list[i]-20)
	
	out = pd.concat([pd.DataFrame(cet),pd.DataFrame(T_list)],axis=1)
	out.columns = ['CET [-]','T [°C]']
	
	return out

def c_thermal_expansion_coef_EC_irr(aggregates='sil'):
	""" returns a table coefcient of thermal expansion (CET) for concrete, with reference to the length at 20°C, for direct input in Abaqus
	accord. EN 1992-1-2 (pp. 26)
	Note: in abaqus thermal strains are defined as total expansion from a reference temperature
	See abaqus manual: http://dsk.ippt.pan.pl/docs/abaqus/v6.13/books/usb/default.htm?startat=pt05ch26s01abm52.html
	Irrecoverable properties based on: Kabala, 2019 (PhD thesis) Figure 3.2
	
	Parameters
	----------	
	aggregates : String
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	Returns
	-------
	out : pd.DataFrame
		Concrete thermal expansion data in 3 columns for Abaqus input	| exp coef | Temperature [°C] | Max. temperature [°C] |
	"""
	
	# !!! Hardcoded import from excel file, to be coded later !!!
	return df_readFromExcel('input\concrete_irr_thermal_expansion.xlsx','Abaqus')['Abaqus'].reset_index()


def Abaqus_CDP_parameters():
	""" Generates """
	dilation_angle = [38,38,38] # default value
	eccentricity = [0.1,0.1,0.1] # default value
	fb0fc0 = [1.16,1.3,1.6]
	K = [0.666,0.666,0.666] # default value
	viscosity = [0,0,0]
	T_list = [20,300,700]
	
	out = pd.concat([pd.DataFrame(dilation_angle),pd.DataFrame(eccentricity),pd.DataFrame(fb0fc0),pd.DataFrame(K),pd.DataFrame(viscosity),pd.DataFrame(T_list)],axis=1)
	out.columns = ['dilat.','ecc.','fb/fc','K','visc.','T']
	return out


#################################################
### Steel strength and deformatoin properties ###
#################################################
	
def s_stress_strain_EC_T(fy,Es):
	""" obtain datapoints for steel stress-strain relationship at elevated temperatures, for direct input in Abaqus, in range 20 - 1100 °C
	accord. EN 1992-1-2 (pp. 23)
	
	Note: softening is not yet included
	
	Parameters
	----------
	fy : float
		steel tensile strength (at 20°C) [MPa]
	
	Es : float
		slope of the linear elastic range [MPa]
	
	Returns
	-------
	s_out : pd.DataFrame
		Stress-strain data in 3 columns for Abaqus input	| stress [MPa] | pl. strain [-] | Temperature [°C] |
	
	e_out : pd.DataFrame
		Elasticity data in 3 columns for Abaqus input	| Young's mod [MPa] | Poisson rat. [-] | Temperature [°C] |
	"""
	
	n = 10 # number of data points in ascending branch (min. 5 recommended)
	n2 = 50
	
	T_list = np.array([20,100,200,300,400,500,600,700,800,900,1000,1100])
	kfy_list, ksp_list, kE_list = MechanicalTools.s_strength_retention_EC(T_list)
	
	sig_data=[]; ep_data=[]
	
	# fixed strain values see figure 3.3, pp. 23
	e_sy = 0.02 # [-]
	e_st = 0.15 # [-]
	e_su = 0.20 # [-]
	
	fy_T_list = fy*kfy_list # [MPa] reduced yield strength
	fsp_T_list = fy*ksp_list # [MPa] proportional limit fsp
	E_T_list = Es*kE_list # [MPa] (reduced) slope of the linear elastic range
	e_sp_list = fsp_T_list/E_T_list # [-] total strain at proportional limit fsp
	
	c_list = ((fy_T_list - fsp_T_list)**2)/(E_T_list*(e_sy-e_sp_list) - 2*(fy_T_list - fsp_T_list))
	a_list = np.sqrt((e_sy - e_sp_list)*(e_sy - e_sp_list + c_list/E_T_list))
	b_list = np.sqrt(c_list*c_list + c_list*(e_sy - e_sp_list)*E_T_list)
	
	# generate data points for each temperature in T_list
	for T,a,b,c,fyt,f_sp,e_sp,E in zip(T_list,a_list,b_list,c_list,fy_T_list,fsp_T_list,e_sp_list,E_T_list):
		e_list = arangeBias(e_sp,e_sy,n+1,10)
		s_list = f_sp - c + (b/a)*np.sqrt((a**2 - (e_sy - e_list)**2))		
		
		e_el_list = s_list/E # elastic strain
		e_pl_list = e_list - e_el_list # plastic strain
		e_pl_list[0] = 0
		
		s_list = np.append(s_list,[fyt,5,5])
		e_pl_list = np.append(e_pl_list,[e_st - fyt/E,e_su-5/E,0.21])
		
		s_list,e_pl_list = regalurize_fp(s_list,e_pl_list,n=n2)
		
		sig_data.append(s_list)
		ep_data.append(e_pl_list)
	
	# repackage stress strain data for output
	sig_out = np.array(sig_data).flatten() # [MPa]
	ep_out = np.array(ep_data).flatten() # [-]
	T_out = np.repeat(T_list,n2) # [°C]
	
	s_out = pd.concat([pd.DataFrame(sig_out),pd.DataFrame(ep_out),pd.DataFrame(T_out)],axis=1)
	s_out.columns = ['s [MPa]','e_pl [-]','T [°C]']
	
	# repackage elasticity data for output
	E_out = np.array(E_T_list)
	pois_out = np.ones(T_list.size)*0.3
	
	e_out = pd.concat([pd.DataFrame(E_out),pd.DataFrame(pois_out),pd.DataFrame(T_list)],axis=1)
	e_out.columns = ['E [MPa]','v [-]','T [°C]']

	return s_out, e_out


def s_thermal_expansion_coef_EC():
	""" returns a table coefcient of thermal expansion (CET) for reinforcing steel, with reference to the length at 20°C, for direct input in Abaqus
	accord. EN 1992-1-2 (pp. 29)
	Note: in abaqus thermal strains are defined as total expansion from a reference temperature
	See abaqus manual: http://dsk.ippt.pan.pl/docs/abaqus/v6.13/books/usb/default.htm?startat=pt05ch26s01abm52.html
	
	Returns
	-------
	out : pd.DataFrame
		Dataframe with 2 columns for Abaqus input	 | CET [-] | Temperature [°C] | 
	"""
	
	T_list = np.arange(20,1101,1)
	e_th = MechanicalTools.s_thermal_strain_EC(T_list)
	e_th[0]=0 # set first value equal to 0, EC2 analytical expression is exactly non-zero at 20°C
	
	cet = np.zeros(len(T_list))
	for i in range(0,len(T_list)):
		if i>0:
			cet[i]=(e_th[i]-e_th[0])/(T_list[i]-20)
	
	out = pd.concat([pd.DataFrame(cet),pd.DataFrame(T_list)],axis=1)
	out.columns = ['CET [-]','T [°C]']
	
	return out


##############################################################
### Prestressing steel strength and deformatoin properties ###
##############################################################

def p_stress_strain_EC_T(fp,Ep,softening=False,return_T_list=False):
	""" obtain datapoints for concrete stress-strain relationship at elevated temperatures, for direct input in Abaqus, in range 20 - 1100 °C
	accord. EN 1992-1-2 (pp. 23)
	
	Parameters
	----------
	fp : float
		prestressing steel tensile strength (at 20°C) [MPa]
	
	Ep : float
		slope of the linear elastic range [MPa]

	Returns
	-------
	s_out : pd.DataFrame
		Stress-strain data in 3 columns for Abaqus input	| stress [MPa] | pl. strain [-] | Temperature [°C] |
	
	e_out : pd.DataFrame
		Elasticity data in 3 columns for Abaqus input	| Young's mod [MPa] | Poisson rat. [-] | Temperature [°C] |
	"""
	
	n = 10 # number of data points in ascending branch (min. 5 recommended)
	n2 = 50 # number of regularized points
	
	T_list = np.array([20,100,200,300,400,500,600,700,800,900])
	kfy_list, ksp_list, kEp_list = MechanicalTools.p_strength_retention_EC(T_list)
	beta = 1 # [-] 
	
	sig_data=[]; ep_data=[]
	
	# fixed strain values see figure 3.3, pp. 23
	e_py = 0.02 # [-]
	e_pt = 0.05 # [-]
	e_pu = 0.1 # [-] 
	
	fy_T_list = fp*kfy_list*beta # [MPa] reduced yield strength
	fsp_T_list = fp*ksp_list*beta # [MPa] proportional limit fsp
	E_T_list = Ep*kEp_list # [MPa] (reduced) slope of the linear elastic range
	e_sp_list = fsp_T_list/E_T_list # [-] total strain at proportional limit fsp
	
	c_list = ((fy_T_list - fsp_T_list)**2)/(E_T_list*(e_py-e_sp_list) - 2*(fy_T_list - fsp_T_list))
	a_list = np.sqrt((e_py - e_sp_list)*(e_py - e_sp_list + c_list/E_T_list))
	b_list = np.sqrt(c_list*c_list + c_list*(e_py - e_sp_list)*E_T_list)
	
	# generate data points for each temperature in T_list
	for T,a,b,c,fy_T,f_sp,e_sp,E in zip(T_list,a_list,b_list,c_list,fy_T_list,fsp_T_list,e_sp_list,E_T_list):
		e_list = arangeBias(e_sp,e_py,n+1,10)
		s_list = f_sp - c + (b/a)*np.sqrt((a**2 - (e_py - e_list)**2))
		
		e_el_list = s_list/E # elastic strain
		e_pl_list = e_list - e_el_list # plastic strain
		e_pl_list[0] = 0
		
		if softening:
			s_list = np.append(s_list,[fy_T,50,50])
			e_pl_list = np.append(e_pl_list,[e_pt - fy_T/E,e_pu-50/E,0.11])
		
		s_list,e_pl_list = regalurize_fp(s_list,e_pl_list,n=n2)
		
		sig_data.append(s_list)
		ep_data.append(e_pl_list)
	
	# repackage stress strain data for output
	sig_out = np.array(sig_data).flatten() # [MPa]
	ep_out = np.array(ep_data).flatten() # [-]
	T_out = np.repeat(T_list,n2) # [°C]
	
	s_out = pd.concat([pd.DataFrame(sig_out),pd.DataFrame(ep_out),pd.DataFrame(T_out)],axis=1)
	s_out.columns = ['s [MPa]','e_pl [-]','T [°C]']
	
	# repackage elasticity data for output
	E_out = np.array(E_T_list)
	pois_out = np.ones(T_list.size)*0.3
	
	e_out = pd.concat([pd.DataFrame(E_out),pd.DataFrame(pois_out),pd.DataFrame(T_list)],axis=1)
	e_out.columns = ['E [MPa]','v [-]','T [°C]']
	
	if return_T_list:
		return s_out, e_out, T_list
	else:
		return s_out, e_out


def p_thermal_expansion_coef_EC():
	""" returns a table coefcient of thermal expansion (CET) for prestressing steel, with reference to the length at 20°C, for direct input in Abaqus
	accord. EN 1992-1-2 (pp. 29)
	Note: in abaqus thermal strains are defined as total expansion from a reference temperature
	See abaqus manual: http://dsk.ippt.pan.pl/docs/abaqus/v6.13/books/usb/default.htm?startat=pt05ch26s01abm52.html

	Returns
	-------
	out : pd.DataFrame
		Dataframe with 2 columns for Abaqus input	 | CET [-] | Temperature [°C] | 
	"""
	
	T_list = np.arange(20,1101,1)
	e_th = MechanicalTools.p_thermal_strain_EC(T_list)
	e_th[0]=0 # set first value equal to 0, EC2 analytical expression is exactly non-zero at 20°C
	
	cet = np.zeros(len(T_list))
	for i in range(0,len(T_list)):
		if i>0:
			cet[i]=(e_th[i]-e_th[0])/(T_list[i]-20)

	out = pd.concat([pd.DataFrame(cet),pd.DataFrame(T_list)],axis=1)
	out.columns = ['CET [-]','T [°C]']
	
	return out


########################################################
### AUX : Material properties in fib Model Code 2010 ###
########################################################
def c_tensile_strength_fib(fcm):
	""" calculate concrete mean tensile strength, accord. fib MC 2010 (pp. 118)

	Parameters
	----------
	fcm : float
		concrete mean compressive strength [MPa]
	
	Returns
	-------
	fctm : float
		concrete mean tensile strenght [MPa]
	"""
	
	fck = fcm -8
	fctm = 0.3*fck**(2/3) if fck <= 50 else 2.12*np.log(1+0.1*(fcm))
	return fctm


def c_fracture_energy_fib(fcm):
	""" calculate concrete fracture energy, accord. fib MC 2010 (pp. 120)
	
	Parameters
	----------
	fcm : float
		concrete mean compressive strength [MPa]
	
	Returns
	-------
	Gf : float
		concrete fracture energy (energy required to propagate a tensile crack of unit area) [N/m]
	"""
	return 73*fcm**(0.18)


###################
### AUX METHODS ###
###################

def regularize_fct(data, T_list, n=25):
	w_max = max(data['w [mm]'])
	w_out = arangeBias(0,w_max,n,20)
	
	s_data=[]; w_data=[]
	
	for T in T_list:
		data_T = data.loc[data['T [°C]'] == T]
		s_list = np.array(data_T['s [MPa]'])
		w_list = np.array(data_T['w [mm]'])
		
		s_list = np.append(s_list,s_list[-1])
		w_list = np.append(w_list,w_max)
		
		s_interpol = interp1d(w_list,s_list)
		s_out = s_interpol(w_out)
		
		s_data.append(s_out)
		w_data.append(w_out)
		
	s_df = np.array(s_data).flatten()
	w_df = np.array(w_data).flatten()
	T_df = np.repeat(T_list,n)
	
	out = pd.concat([pd.DataFrame(s_df),pd.DataFrame(w_df),pd.DataFrame(T_df)],axis=1)
	out.columns = ['s [MPa]','w [mm]','T [°C]']
	return out


def regalurize_fp(sig,ep,n=10):
	s_interpol = interp1d(ep,sig)
	ep_out = np.linspace(min(ep), max(ep),n)
	return s_interpol(ep_out), ep_out


def arangeBias(start,end,n,bias):
	""" create an array of numbers using a certain bias (similar to meshing bias in Abaqus)
	when bias = 1, behaves like np.arange(start,end,n)

	Parameters
	----------
	start, end : float
		start and end value of array
	
	n : int
		number of items in array, including 'start' and 'end'
		
	bias : float
		scale difference between first and last interval
	
	Returns
	-------
	out : np.array
		array of length n, in which the first interval is <bias> times the last interval
	
	Example use
	-----------
	>>> import numpy as np
	>>> from TTH_util import arangeBias
	>>> print(arangeBias(10,20,5,2))
	[10.   11.71018456   13.8648821    16.57963087   20.]
	"""
	
	bias = bias**(1/(n-2))
	
	intervals = []
	for i in range(0,n-1): intervals.append(bias**i)
	intervals_scaled = np.array(intervals)/sum(intervals)
	
	points = np.array([0])
	s = 0
	for i in intervals_scaled:
		s = s + i
		points = np.append(points,[s])
	
	return points*(end-start)+start



#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":
	
	SW_test = 2

	print("testing")
	
	if SW_test==1:
		import matplotlib.pyplot as plt
		fy=500
		Es=210000
		df_s, df_E = s_stress_strain_EC_T(fy,Es)
		for T in df_s['T [°C]'].unique():
			data = df_s.loc[df_s['T [°C]'] == T]
			E = df_E.loc[df_E['T [°C]'] == T]['E [MPa]']
			s = np.array(data['s [MPa]'])
			epl = data['e_pl [-]']
			eel = np.array(s/float(E))
			etot = eel+epl
			
			s = np.append([0], s)
			etot = np.append([0],etot)
			
			plt.plot(etot,s)
	
	if SW_test == 2:
		a = c_thermal_expansion_coef_EC_irr()



#__author__ = "magnelPy"
#__email__= "ruben.vancoile@ugent.be"
#__date__= "2018-11-21"

####################
## MODULE IMPORTS ##
####################

import numpy as np
import pandas as pd
import os, shutil
import openpyxl
from openpyxl import load_workbook #resolves issue with writing data to the correct Excel-sheet
import time

##############
## FUNCTION ##
##############

## print pd.DataFrame to *.xlsx ##
##################################

def df_writeToExcel(data,name,sheet,SW_overwrite=True):
	#--------------------------------------------------------------------------------------------------------
	# Print single pd.DataFrame or list of pd.DataFrame to specified excelfile and sheet
	# Input:
	#	data : pd.DataFrame / list of pd.DataFrame (df/list)
	#	name : (str) path of excel file (can be relative to working dir)
	# 	sheet : name / list of names (str / list) for the excel-sheets
	#	SW_overwrite : (boolean) overwrite existing excel or not
	#--------------------------------------------------------------------------------------------------------

	# remove '.xlsx' from name if specified
	if name[-5:]=='.xlsx': name=name[:-5]

	# set all input to list
	if isinstance(data,pd.DataFrame): data=[data]; sheet=[sheet]

	# check if excel path exists - SW_overwrite to True if non-existent
	if not os.path.isfile(name+'.xlsx'): SW_overwrite=True

	# print data to excel
	dfList_writeToExcel(data,name,sheet,SW_overwrite)

## read multiple sheets from *.xlsx ##
######################################

def df_readFromExcel(name,sheet=None):
	#--------------------------------------------------------------------------------------------------------
	# Return Dict of pd.DataFrames with data *.xlsx worksheets, worksheet names as Dict keys
	# Input:
	#	name : (str) path of excel sheet (can be relative to working dir)
	# 	sheet : name / list of names (str / list) for the excel-sheets; if None : all sheets are parsed
	#--------------------------------------------------------------------------------------------------------

	# read full *.xlsx file (suboptimal)
	xlsx=pd.ExcelFile(name)

	# syntax formatting
	if not isinstance(sheet,list):
		if sheet is None:
			sheet = xlsx.sheet_names
		else: sheet=[sheet]

	# read worksheets and store in out-Dict
	out={}
	for worksheet in sheet:
		out[worksheet]=pd.read_excel(xlsx,worksheet,index_col=0)

	return out

## clean folders ##
###################

def RemoveFolderData(folder):

	for the_file in os.listdir(folder):
		file_path=os.path.join(folder, the_file)
		if os.path.isfile(file_path):
			os.unlink(file_path)


##################
## AUX FUNCTION ##
##################

def dfList_writeToExcel(mytotallist,name,sheet,SW_overwrite=True):
	#--------------------------------------------------------------------------------------------------------
	# Print list of pd.DataFrame to specified excelfile and sheet
	# Input:
	#	mytotallist : (list) list of pd.DataFrames
	#	name : (str) path of excel sheet (can be relative to working dir)
	# 	sheet : (list) list of names (str) for the excel-sheets
	# Note1: overwrites existing excelfile and will throw an error when an existing file is open
	# Note2: renamed function Print_DataFrame() (rvcpy)
	# Known issue: duplicate name will result in additional sheets printed
	#--------------------------------------------------------------------------------------------------------
	writer=pd.ExcelWriter(name+'.xlsx',engine='openpyxl')
	if not SW_overwrite: writer.book=load_workbook(name+'.xlsx')
	for n,df in enumerate(mytotallist):
		Single_Print_DataFrame(df,writer,sheet[n])
	writer.save()
	writer.close()

def Single_Print_DataFrame(df,writer,sheet):
	#--------------------------------------------------------------------------------------------------------
	# Print single pd.DataFrame to specified excelfile and sheet
	# Input:
	#	df : pd.DataFrame
	#	writer : pd.ExcelWriter object
	# 	sheet : (str) name of excel-sheet
	# Note: overwrites existing excelfile and will throw an error when an existing file is open
	#--------------------------------------------------------------------------------------------------------
	df.to_excel(writer,sheet_name=sheet)

def TimeStamp():
	""" Provides unique timestamp

	Returns
	-------
	timestamp : str with unique timestamp
	 
	"""
	return str(int(time.time()*10**6))

#####################
## LEGACY FUNCTION ##
#####################

def writeToExcel(df,filename,sheetname=None):
	#--------------------------------------------------------------------------------------------------------
	# Print list of pd.DataFrame to specified excelfile and sheet
	# Input:
	#	df : pd.DataFrame
	#	name : (str) path of excel sheet (can be relative to working dir)
	# 	sheet : (str) name of excel-sheet
	# Note1: adds to existing excelfile and will throw an error when file does not exist
	# acknowledgement: TTH 11.2018 - function in GenerateRandom.py
	#--------------------------------------------------------------------------------------------------------
    writer = pd.ExcelWriter(filename+'.xlsx',engine='openpyxl')
    writer.book = load_workbook(filename+'.xlsx')
    if sheetname is not None: Single_Print_DataFrame(df,writer,sheetname)
    Single_Print_DataFrame(df,writer,'Sheet1')
    writer.save()

def Print_DataFrame(mytotallist,name,sheet,SW_overwrite=True):
	#--------------------------------------------------------------------------------------------------------
	# Print list of pd.DataFrame to specified excelfile and sheet
	# Input:
	#	mytotallist : (list) list of pd.DataFrames
	#	name : (str) path of excel sheet (can be relative to working dir)
	# 	sheet : (list) list of names (str) for the excel-sheets
	# Note1: overwrites existing excelfile and will throw an error when an existing file is open
	# Note2: renamed function Print_DataFrame() (rvcpy)
	#--------------------------------------------------------------------------------------------------------
	writer=pd.ExcelWriter(name+'.xlsx')
	for n,df in enumerate(mytotallist):
		Single_Print_DataFrame(df,writer,sheet[n])
	writer.save()
	writer.close()

#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":

	## df_writeToExcel test ##
	if True:
		data=pd.DataFrame(0,index=['A','B'],columns=[1,2])
		name='test.xlsx'
		sheet='fourth'
		df_writeToExcel(data,name,sheet,False)


	## df_readFromExcel test ##
	if False:
		path="C:\\Users\\rvcoile\\Google Drive\\Research\\Codes\\Specific\\specific\\Nload_CDF - kopie.xlsx"
		sheet=['Iqbal_Fx','VanCoile_Fx']

		data=df_readFromExcel(path,sheet)

		print(data['VanCoile_Fx'])


from tkinter import filedialog
from tkinter import *
import pickle
import matplotlib.pyplot as plt
import os

cwd=os.getcwd()
print(cwd)


root = Tk()
root.filename =  filedialog.askopenfilename(initialdir = cwd,title = "Select file")

fig_handle = pickle.load(open(root.filename,'rb'))
print(fig_handle)
print('blabalba')
print(type(fig_handle))
plt.show()


#__author__ = "magnelPy"
# test 1

#--------------------------------------------------------------------------------------------------------
# Standard formatting stochastic variables
#--------------------------------------------------------------------------------------------------------

####################
## MODULE IMPORTS ##
####################

import numpy as np
import scipy as sp
from magnelPy import statFunc as sf

##############
## FUNCTION ##
##############

def createStochVar(name='',dist='normal',mean=0.,std=1.,dim='[-]',comment='',theta1=None,theta2=None):
	## standard formatting of stochastic variables ##
	# name = name of stochastic variable
	# dist = distribution type
	# 'normal';'lognormal';'mixedlognormal';'gumbel';'deterministic';'beta';'gamma';'uniform';'weibull';'lognormal_truncated'
	# mean = mean value
	# std = standard deviation
	# dim = dimension
	# comment = free notes
	# theta1 = first additional parameter - when appropriate - can be improved
	# theta2 = second additional parameter - when appropriate - can be improved
	return {'name':name,'dist':dist,'dim':dim,'m':mean,'s':std,'info':comment,'theta1':theta1,'theta2':theta2}

def fx(varDict,xArray,SW_log=False):
	DistType=varDict['dist']
	if DistType=='normal':
		return sf.f_Normal(xArray,varDict['m'],varDict['s'],SW_log)
	if DistType=='lognormal':
		return sf.f_Lognormal(xArray,varDict['m'],varDict['s'])
	if DistType=='mixedlognormal':
		return sf.f_MixedLN(xArray,varDict['mi'],varDict['si'],varDict['Pi'])
	if DistType=='gumbel':
		return sf.f_Gumbel(xArray,varDict['m'],varDict['s'])
	if DistType=='beta':
		return sf.f_Beta_ab(xArray,varDict['theta1'],varDict['theta2'],varDict['m'],varDict['s'],SW_log)
	if DistType=='gamma':
		return sf.f_Gamma(xArray,varDict['m'],varDict['s'])
	if DistType=='uniform':
		return sf.f_Uniform(xArray,varDict['theta1'],varDict['theta2'])
	if DistType=='weibull':
		return sf.f_Weibull(xArray,varDict['m'],varDict['s'])
	if DistType=='lognormal_truncated':
		return sf.f_Lognormal_truncated(xArray,varDict['m'],varDict['s'],varDict['theta1'])

def Fx(varDict,xArray):
	DistType=varDict['dist']
	if DistType=='normal':
		return sf.F_Normal(xArray,varDict['m'],varDict['s'])
	if DistType=='lognormal':
		return sf.F_Lognormal(xArray,varDict['m'],varDict['s'])
	if DistType=='mixedlognormal':
		return sf.F_MixedLN(xArray,varDict['mi'],varDict['si'],varDict['Pi'])
	if DistType=='gumbel':
		return sf.F_Gumbel(xArray,varDict['m'],varDict['s'])
	if DistType=='beta':
		return sf.F_Beta_ms(xArray,varDict['m'],varDict['s'],varDict['theta1'],varDict['theta2'])
	if DistType=='gamma':
		return sf.F_Gamma(xArray,varDict['m'],varDict['s'])
	if DistType=='uniform':
		return sf.F_Uniform(xArray,varDict['theta1'],varDict['theta2'])
	if DistType=='weibull':
		return sf.F_Weibull(xArray,varDict['m'],varDict['s'])
	if DistType=='lognormal_truncated':
		return sf.F_Lognormal_truncated(xArray,varDict['m'],varDict['s'],varDict['theta1'])

def Finvx(varDict,rArray):
	DistType=varDict['dist']
	if DistType=='normal':
		return sf.Finv_Normal(rArray,varDict['m'],varDict['s'])
	if DistType=='lognormal':
		return sf.Finv_Lognormal(rArray,varDict['m'],varDict['s'])
	if DistType=='mixedlognormal':
		return sf.Finv_MixedLN(rArray,varDict['mi'],varDict['si'],varDict['Pi'])
	if DistType=='gumbel':
		return sf.Finv_Gumbel(rArray,varDict['m'],varDict['s'])
	if DistType=='beta':
		return sf.Finv_Beta_ms(rArray,varDict['m'],varDict['s'],varDict['theta1'],varDict['theta2'])
	if DistType=='gamma':
		return sf.Finv_Gamma(rArray,varDict['m'],varDict['s'])
	if DistType=='uniform':
		return sf.Finv_Uniform(rArray,varDict['theta1'],varDict['theta2'])
	if DistType=='weibull':
		return sf.Finv_Weibull(rArray,varDict['m'],varDict['s'])
	if DistType=='lognormal_truncated':
		return sf.Finv_Lognormal_truncated(rArray,varDict['m'],varDict['s'],varDict['theta1'])

## RF transformation ##
#######################

def RF_fxN_FxN(x,muN,sigN):
    uN=(x-muN)/sigN
    FxN=sf.F_Normal(uN,0,1)
    fxN=1/sigN*sf.f_Normal(uN,0,1)
    return FxN,fxN

def RF_deviation(parN,x,varDict):
    muN=parN[0];sigN=parN[1]
    fxx=fx(varDict,x); Fxx=Fx(varDict,x)
    FxN,fxN=RF_fxN_FxN(x,muN,sigN)
    return Fxx-FxN,fxx-fxN

def RF_muN_sigN_solver(x,varDict):
    out=sp.optimize.fsolve(RF_deviation,[varDict['m'],varDict['s']],args=(x,varDict))
    muN=out[0];sigN=out[1]
    return muN,sigN

def RF_transform(x,varDict):
    muN,sigN=RF_muN_sigN_solver(x,varDict)
    return (x-muN)/sigN,muN,sigN

    
##################
## AUX FUNCTION ##
##################


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":

	rebar = 0.01  #rebar diameter [m]
	dist = 0.1  #distance between rebars [m]
	w = 1.    #slab width [m]	

	As=createStochVar(dist='normal',mean=0.25*(np.pi)*rebar**2*(w/dist)*1.02,std=0.02*0.25*(np.pi)*rebar**2*(w/dist)*1.02,dim='[m2]',name='As [m2]')
	fck=30; Vfc=0.15 # [MPa]; [-]
	fc=createStochVar(dist='lognormal',mean=fck/(1-2*Vfc),std=fck/(1-2*Vfc)*Vfc,dim='[MPa]',name='fc20 [MPa]')

	StochVarDict={'As':As,'fc':fc}

	nameList=[StochVarDict[key]['name'] for key in StochVarDict.keys()]

	print(StochVarDict.keys())    



"""Fire definition

Notes
-----
The FireCurve module supports definition of temperature-time curves for SFE.
"""

#############
## IMPORTS ##
#############

import numpy as np
import math as m
from scipy.interpolate import interp1d

############
## MODULE ##
############

def ISO834(time):
	""" Return ISO834 gas temperature at specified times

	Parameters
	----------
	time :	np.array
		array of time instants [min]

	Returns
	-------
	fire :	np.array
		array of gas temperatures for time [C]

	Reference
	---------
	EN 1991-1-2:2002. Eurocode 1: Actions on structures - Part 1-2:
		General actions - Actions on structures exposed to fire, 3.2.1 (p24) 

	Examples
	--------
	>>> import numpy as np
	>>> import magnelPy.SFE as sfe
	>>> time = np.arange(0,120+1,1)
	>>> fire = sfe.FireCurve.ISO834(time)
	>>> print(np.array([time[[0,30,60,90,120]],
		np.around(fire[[0,30,60,90,120]],0)]))
	[[   0.   30.   60.   90.  120.]
 	[  20.  842.  945. 1006. 1049.]]
	"""
	return 20+345*np.log10(8*time+1)


def ASTM(time):
	""" Return ASTM-E119 gas temperature at specified times

	Parameters
	----------
	time :	np.array
		array of time instants [min]

	Returns
	-------
	fire :	np.array
		array of gas temperatures for time [C]

	Reference
	---------
	ASTM E119-00a: Fire Tests of Building Construction and Materials
	via https://www.nrc.gov/docs/ML0516/ML051640380.pdf

	"""
	t_list = np.array([0,5,10,20,30,60,90,120,180,240,300,360,420,480]) # [min]
	T_list = np.array([68,1000,1300,1462,1550,1700,1792,1850,1925,2000,2075,2150,2225,2300]) # [°F]
	T_list = (T_list-32)*5/9 # conversion °F -> °C
	T_data = interp1d(t_list,T_list)
	return T_data(time)


def EuroCodeParametric(time, openingFactor, fireLoadDensity, thermalInertia=1450, length=10, width=10, height=3, t_lim = 20, Reitgruber=False):
	""" Return Eurocode Parametric fire gas temperature at specified times

	Parameters
	----------
	time :	np.array
		array of time instants [min]

	openingFactor : float
		opening factor (typically) between 0.02 and 0.2 [m1/2]

	fireLoadDensity : float
		fire load density related to the surface area [MJ/m2]

	thermalInertia : float
		thermal interia of the compartment, typically between 100 and 2200 [J/m2 s1/2 K] (default = 1450 J/m2 s1/2 K)

	length, width, height : float
		dimensions of a rectangular compartment [m] (default = 10/10/3 m)

	t_lim : float
		factor which indicates the fire growth rate [min] (default = 20 min)
		
	Reitgruber : Boolean
		Take into account correction proposed by Reitguber et al. * (default = False)

	Returns
	-------
	fire :	np.array
		array of gas temperatures for time [C]

	Reference
	---------
	EN 1991-1-2:2002. Eurocode 1: Actions on structures - Part 1-2:
	  Annex A - Parametric temperature-time curves (p30-32)
   *Reitgruber, S., Blasi, C. Di and Franssen, J. (2006) ‘Some Comments on the Parametric Fire Model of Eurocode 1’, 
	    Conference on Fire in Enclosures, (January 2006), pp. 1–25.
	
	Examples
	--------
	>>> import numpy as np
	>>> import magnelPy.SFE as sfe
	>>> time = np.arange(0,120+1,1)
	>>> fire = sfe.FireCurve.EuroCodeParametric(time,0.05,500,1450,10,10,3)
	"""

	# calculation of other parameters required to determine the shape of the heating a cooling phase curves
	A_f = length*width # [m2] floor area
	A_t = 2*A_f + 2*height*(length+width) # [m2] total compartment surface area
	qt_d = fireLoadDensity*A_f/A_t # [MJ/m2] design value of the fire load density related to the total surface area A_t
	Of_lim = 0.0001*qt_d/(t_lim/60) # [m1/2] adjusted opening factor in case of fuel controlled fire
	t_max = max(t_lim/60, 0.0002*qt_d/openingFactor) # [hours] time at which the maximum temperature in the heating phase occurs
	if Reitgruber: 
		Of_lim = 0.00014*qt_d/(t_lim/60)
		t_max = max(t_lim/60, 0.00014*qt_d/openingFactor)
	Lambda = ((openingFactor/thermalInertia)**2)/((0.04/1160)**2) # [-] time scale parameter
	k = 1 + ((openingFactor-0.04)/0.04)*((qt_d-75)/75)*((1160-thermalInertia)/1160) if (openingFactor > 0.04 and qt_d > 75 and thermalInertia <1160) else 1
	Lambda_lim = k*((Of_lim/thermalInertia)**2)/((0.04/1160)**2) # [-] time scale parameter in case of fuel controlled fire
	Lambda_use = Lambda_lim if t_max==t_lim/60 else Lambda
	t_max_st = t_max*Lambda if t_max > t_lim/60 else t_max*Lambda_lim
	theta_max = 20+1325*(1-0.324*m.exp(-0.2*t_max*Lambda_use)-0.204*m.exp(-1.7*t_max*Lambda_use)-0.472*m.exp(-19*t_max*Lambda_use)) # time temperature curve for heating phase (EN 1991-1-2 - Formula A.1)
	x = 1 if t_max>t_lim/60 else t_lim/60*Lambda/(t_max*Lambda_use)
	
	# calcalate tempertures [°C]
	temp_list = []
	for t in time:
		if t/60 <= t_max: # heating phase
			temp_list.append(20+1325*(1 - 0.324*m.exp(-0.2*(t/(60/Lambda_use))) - 0.204*m.exp(-1.7*(t/(60/Lambda_use))) - 0.472*m.exp(-19*(t/(60/Lambda_use)))))
		else: # cooling phase
			if t_max_st < 0.5:
				temp_list.append(max(20,theta_max - 625*(t/60*Lambda - t_max_st*x))) # (EN 1991-1-2 - Formula A.11a)
			else:
				if t_max_st < 2:
					temp_list.append(max(20,theta_max - 250*(3 - t_max_st)*(t/60*Lambda - t_max_st*x))) # (EN 1991-1-2 - Formula A.11b)
				else:
					temp_list.append(max(20,theta_max - 250*(t/60*Lambda - t_max_st*x))) # (EN 1991-1-2 - Formula A.11c)
	return temp_list

def EPFC(time, openingFactor, fireLoadDensity, thermalInertia=1450, length=10, width=10, height=3, t_lim = 20, Reitgruber=False):
	""" Return Eurocode Parametric fire gas temperature at specified times - vectorized function

	Parameters
	----------
	time :	np.array (m,)
		array of time instants [min]

	openingFactor : np.array (n,)
		array of opening factor (typically) between 0.02 and 0.2 [m1/2]

	fireLoadDensity : np.array (n,)
		array of fire load density related to the surface area [MJ/m2]

	thermalInertia : float or np.array (n,)
		thermal interia of the compartment, typically between 100 and 2200 [J/m2 s1/2 K] (default = 1450 J/m2 s1/2 K)

	length, width, height : float or np.array (n,)
		dimensions of a rectangular compartment [m] (default = 10/10/3 m)

	t_lim : float or np.array (n,)
		factor which indicates the fire growth rate [min] (default = 20 min)
		
	Reitgruber : Boolean
		Take into account correction proposed by Reitguber et al. * (default = False)

	Returns
	-------
	fire :	np.array(n,m)
		array of gas temperatures for time [C]

	Reference
	---------
	EN 1991-1-2:2002. Eurocode 1: Actions on structures - Part 1-2:
	  Annex A - Parametric temperature-time curves (p30-32)
	Reitgruber, S., Blasi, C. Di and Franssen, J. (2006) ‘Some Comments on the Parametric Fire Model of Eurocode 1’, 
	    Conference on Fire in Enclosures, (January 2006), pp. 1–25.
	
	Examples
	--------
	import numpy as np
	import magnelPy.SFE as sfe
	time = np.arange(0,120+1,1)
	O=np.array([0.04,0.05])
	qf=np.array([600,700])
	fire = sfe.FireCurve.EPFC(time,O,qf)
	timeprint=[30,60,90,120]
	for t in timeprint:
    	print("At {0:.0f} min, the temperature is {1:.0f} degrees C for curve1 and {2:.0f} degrees for curve2.".format(t,fire[0,t],fire[1,t]))
	>>> At 30 min, the temperature is 784 degrees C for curve1 and 841 degrees for curve2.
	>>> At 60 min, the temperature is 843 degrees C for curve1 and 857 degrees for curve2.
	>>> At 90 min, the temperature is 651 degrees C for curve1 and 592 degrees for curve2.
	>>> At 120 min, the temperature is 459 degrees C for curve1 and 326 degrees for curve2.
	"""	
	## check np.array
	if not isinstance(openingFactor,np.ndarray) or not isinstance(fireLoadDensity,np.ndarray):
		raise NameError("This function requires np.array (n,) for both the opening factor and fire load density.")

	## Preparatory steps
	A_f = length*width # [m2] floor area
	A_t = 2*A_f + 2*height*(length+width) # [m2] total compartment surface area
	qt_d = fireLoadDensity*A_f/A_t # [MJ/m2] design value of the fire load density related to the total surface area A_t
	if Reitgruber:
		Of_lim = 0.00014*qt_d/(t_lim/60)
		t_max = np.maximum(t_lim/60, 0.00014*qt_d/openingFactor)
	else: 
		Of_lim = 0.0001*qt_d/(t_lim/60) # [m1/2] adjusted opening factor in case of fuel controlled fire
		t_max = np.maximum(t_lim/60, 0.0002*qt_d/openingFactor)
	Lambda = ((openingFactor/thermalInertia)**2)/((0.04/1160)**2) # [-] time scale parameter
	k=1+((openingFactor-0.04)/0.04)*((qt_d-75)/75)*((1160-thermalInertia)/1160)*(openingFactor > 0.04)*(qt_d > 75)*(thermalInertia <1160)
	Lambda_lim = k*((Of_lim/thermalInertia)**2)/((0.04/1160)**2) # [-] time scale parameter in case of fuel controlled fire
	Lambda_use = np.where(t_max<=t_lim/60,Lambda_lim,Lambda)
	t_max_st = t_max*Lambda_use
	theta_max = 20+1325*(1-0.324*np.exp(-0.2*t_max_st)-0.204*np.exp(-1.7*t_max_st)-0.472*np.exp(-19*t_max_st)) # time temperature curve for heating phase (EN 1991-1-2 - Formula A.1)
	x = np.where(t_max>t_lim/60,1,t_lim/60*Lambda/(t_max*Lambda_use))
	
	## temperature-time calculation
	n=len(theta_max)
	time=time[np.newaxis,:]
	Lambda_use=Lambda_use[:,np.newaxis]; Lambda=Lambda[:,np.newaxis]; theta_max=theta_max[:,np.newaxis]
	# heating phase curve
	Tgh=20+1325*(1-0.324*np.exp(-0.2*time/60*Lambda_use)-0.204*np.exp(-1.7*time/60*Lambda_use)-0.472*np.exp(-19*time/60*Lambda_use))
	# cooling phase curve
	xi=[0.5,2]; yi=[625,250]; rate=np.reshape(np.interp(t_max_st,xi,yi),(n,1))
	Tgc=theta_max - rate*(time/60*Lambda - np.reshape(t_max_st*x,(n,1)))
	# assign temperatures, with reference (minimum) temperature of 20 degrees
	Tg=np.maximum(np.where(Tgc>theta_max,Tgh,Tgc),20)
	
	return Tg 

#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":

	print("testing")


# -*- coding: utf-8 -*-
"""
Radiation calcs for FSE / SFE 
"""
import numpy as np


############
## MODULE ##
############

def F_RectangleToCorner(h,w,d):
    """ Viewfactor from exposing rectangle (h x w) to point at distance d from corner

	Parameters
	----------	
	h : float or (m x n) np.array
		height of the radiating rectangle [distance]

	w : float or (m x n) np.array
		width of the radiating rectangle [distance]

	d : float or (m x n) np.array
		distance to the radiating rectangle [distance]

	Returns
	-------
	F : float or np.array
		 viewfactor [-]
	
	Example use
	-----------
    print("Drysdale 2nd ed, p.59")
    alpha=0.6;S=0.8;F=0.106 # data Drysdale
    h=5 # arbitrary choice
    w=S*h; d=np.sqrt(h*w/alpha)
    print("Calculated : ",F_RectangleToCorner(h, w, d))
    print("Listed by Drysdale : ",F)
    >>>Drysdale 2nd ed, p.59
    >>>Calculated :  0.10633951895326263
    >>>Listed by Drysdale :  0.106
	
	Reference
	-----------    
	Formula PGFSE THMT_Theory12b slide 32
	Code as applied in 'External Fire Spread' project
	"""
    
    X=h/d
    Y=w/d
        
    F=1/(2*np.pi)*(X/np.sqrt(1+X**2)*np.arctan(Y/np.sqrt(1+X**2))+Y/np.sqrt(1+Y**2)*np.arctan(X/np.sqrt(1+Y**2)))

    return F

def F_RectangleToCenter(H,W,d):
    """ Viewfactor from exposing rectangle (H x W) to point at distance d from corner

	Parameters
	----------	
	H : float or np.array
		height of the radiating rectangle [distance]

	W : float or np.array
		width of the radiating rectangle [distance]

	d : float or np.array
		distance to the radiating rectangle [distance]

	Returns
	-------
	F : float or np.array
		 viewfactor [-]
	
	Example use
	-----------
    see F_RectangleToCorner
	
	Reference
	-----------    
	Basic THMT; Drysdale 2nd ed., p.58
	"""
    
    return 4*F_RectangleToCorner(H/2.,W/2.,d)

#################
## MODULE-TEST ##
#################

def moduleTest():
    print("\nmagnelPy.SFE Radiation functionality active\n")

#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":
    
    ## case 1
    print("Drysdale 2nd ed, p.59")
    alpha=0.6;S=0.8;F=0.106 # data Drysdale
    h=5 # arbitrary choice
    w=S*h; d=np.sqrt(h*w/alpha)
    print("h = {0}, w = {1}, d = {2}".format(h,w,d))
    print("Calculated : ",F_RectangleToCorner(h, w, d))
    print("Listed by Drysdale : ",F)
    
    ## case 2
    print()
    H=10
    W=S*H
    print("H = {0}, W = {1}, d = {2}".format(H,W,d))
    print("Calculated : ",F_RectangleToCenter(H, W, d))



"""
magnelPy.SAFIRshell
===================
Provides
  1. SAFIR input file generation and modification
  2. run SAFIR from Python

=============== ==========================================================
SAFIR control ~ control.py
==========================================================================
Test            Test function (prints string to screen)
XXXX            XXXX
=============== ==========================================================

"""

### get submodules ###
from magnelPy.SAFIRshell.control import runSAFIR
# allows for direct (anaconda) command line SAFIR runs >> python -c "import magnelPy.SAFIRshell as x; x.runSAFIR()"
# https://towardsdatascience.com/whats-init-for-me-d70a312da583



"""Master file for automatic creation of Abaqus material models for thermal-stress analysis
"""

###############
### IMPORTS ###
###############

import numpy as np
import pandas as pd
from magnelPy import admin
from magnelPy.SFE import ThermalTools
from magnelPy.Abaqus import MechanicalProperties
from magnelPy.Abaqus import ThermalProperties



##########################
### THERMAL PROPERTIES ###
##########################
def abaqus_concrete_properties_th(rho=2400,moisture=0,limit='lower',units='mm',irreversible=False):
	""" Create inputs for temperature dependant concrete thermal properties 
	(density, specific heat and conductivity) required in an Abaqus heat-transfer analysis
	
	Parameters
	----------
	rho : float (optional)
		Concrete density at room temperature [kg/m3] (Default: 2400)
	
	moisture : float/precentage (optional)
		moisture content, percentage of concrete weight [w_%] in range 0 - 3
	
	limit : String (optional)
		'lower' = Siliceous aggregates (default)
		'upper' = Calcareous aggregates
	
	units : String (optional)
		'mm' = outputs results in SI-mm units (default)
		'm'  = outputs results in SI units
	
	irreversible : boolean (optional)
		False = thermal properties in the cooling phase are fully recovered (default)
		True =  thermal properties in the cooling phase are irreversible, requires Abaqus model with subroutine
	
	Returns
	-------
	ABQ_input_concrete_th : .xls file
		 Results are written to workdir
	"""
	
	T_list = np.arange(20,1101,5) # [°C]
	
	if irreversible == False:
		c_density = ThermalTools.c_density_EC_T(T_list,rho) # [kg/m3]
		c_conductivity = ThermalTools.c_conductivity_EC_T(T_list, limit=limit) # [W/m K]
		c_specific_heat = ThermalTools.c_specific_heat_EC_T(T_list,moisture) # [J/kg K]
		
		if units == 'm':
			out_c_density = pd.concat([pd.DataFrame(c_density),pd.DataFrame(T_list)],axis=1)
			out_c_conductivity = pd.concat([pd.DataFrame(c_conductivity),pd.DataFrame(T_list)],axis=1)
			out_c_specific_heat = pd.concat([pd.DataFrame(c_specific_heat),pd.DataFrame(T_list)],axis=1)
			
			out_c_density.columns = ['rho [kg/m3]','T [°C]']
			out_c_conductivity.columns = ['k [W/m K]','T [°C]']
			out_c_specific_heat.columns = ['cp [J/kg K]','T [°C]']
			
		if units == 'mm':
			out_c_density = pd.concat([pd.DataFrame(c_density*1e-12),pd.DataFrame(T_list)],axis=1)
			out_c_conductivity = pd.concat([pd.DataFrame(c_conductivity),pd.DataFrame(T_list)],axis=1)
			out_c_specific_heat = pd.concat([pd.DataFrame(c_specific_heat*1e6),pd.DataFrame(T_list)],axis=1)
			out_c_density.columns = ['rho [ton/mm3]','T [°C]']
			out_c_conductivity.columns = ['k [mW/mm K]','T [°C]']
			out_c_specific_heat.columns = ['cp [mJ/ton K]','T [°C]']
		
		out_list = [out_c_density,out_c_conductivity,out_c_specific_heat]
		sheet_list = ['c density','c conductivity','c specific_heat']
	
	if irreversible == True:
		out_c_density = ThermalProperties.c_density_EC_T_irr(T_list,rho=rho,units=units)
		out_c_conductivity = ThermalProperties.c_conductivity_EC_T_irr(T_list,limit=limit,units=units)
		out_c_specific_heat = ThermalProperties.c_specific_heat_EC_T_irr(T_list,moisture=moisture,units=units)
		
		out_list = [out_c_density,out_c_conductivity,out_c_specific_heat]
		sheet_list = ['c density','c_conductivity','c_specific_heat']
	
	filename = 'output/ABQ_input_concrete_th'
	admin.dfList_writeToExcel(out_list,filename,sheet_list)
	print('writing Abaqus heat transfer inputs for concrete (units in {})'.format(units))


def abaqus_steel_properties_th(rho_s=7850,units='mm'):
	""" Create inputs for temperature dependant steel thermal properties 
	(density, specific heat and conductivity) required in an Abaqus heat-transfer analysis
	
	Parameters
	----------
	rho : float (optional)
		Steel density at room temperature [kg/m3] (Default: 7850)
	
	units : String (optional)
		'mm' = outputs results in SI-mm units (default)
		'm'  = outputs results in SI units
	
	Returns
	-------
	ABQ_input_concrete_th : .xls file
		 Results are written to workdir
	"""
	
	T_list = np.append([20],np.arange(50,1201,50)) # [°C]
	
	s_conductivity = ThermalTools.s_conductivity_EC_T(T_list) # [W/m K]
	s_specific_heat = ThermalTools.s_specific_heat_EC_T(T_list) # [J/kg K]
	
	if units == 'm':
		out_s_density = pd.DataFrame([rho_s])
		out_s_conductivity = pd.concat([pd.DataFrame(s_conductivity),pd.DataFrame(T_list)],axis=1)
		out_s_specific_heat = pd.concat([pd.DataFrame(s_specific_heat),pd.DataFrame(T_list)],axis=1)
		
		out_s_density.columns = ['rho [kg/m3]']
		out_s_conductivity.columns = ['k [W/m K]','T [°C]']
		out_s_specific_heat.columns = ['k [J/kg K]','T [°C]']
	
	if units == 'mm':
		out_s_density = pd.DataFrame([rho_s*1e-12])
		out_s_conductivity = pd.concat([pd.DataFrame(s_conductivity),pd.DataFrame(T_list)],axis=1)
		out_s_specific_heat = pd.concat([pd.DataFrame(s_specific_heat*1e6),pd.DataFrame(T_list)],axis=1)
		
		out_s_density.columns = ['rho [ton/mm3]']	
		out_s_conductivity.columns = ['k [mW/mm K]','T [°C]']
		out_s_specific_heat.columns = ['k [mJ/ton K]','T [°C]']
	
	filename = 'output/ABQ_input_steel_th'
	out_list = [out_s_density,out_s_conductivity,out_s_specific_heat]
	sheet_list = ['s density','s conductivity','s specific_heat']
	admin.dfList_writeToExcel(out_list,filename,sheet_list)
	print('writing Abaqus heat transfer inputs for steel (units in {})'.format(units))


#############################
### MECHANICAL PROPERTIES ###
#############################

def abaqus_concrete_properties_mech(fcm,fct=-1,aggregates='sil',irreversible=False):
	""" Create inputs for temperature dependant concrete mechanical properties 
	(stress-strain curves, Young's modulus, thermal expansion) required in an Abaqus thermal-stress analysis
	
	Parameters
	----------
	fcm : float
		Concrete compressive strength at 20°C [MPa]
	
	fct : float (optional)
		Concrete tensile strength at 20°C [MPa] (default = -1: fct is calculated based on fib Model Code 2010)
		
	aggregates : String (optional)
		'sil'  = Siliceous aggregates (default)
		'calc' = Calcareous aggregates
	
	irreversible : boolean (optional) NOT IMPLEMENTED
		False = mechanical properties in the cooling phase are fully recovered (default)
		True =  mechanical properties in the cooling phase are irreversible, requires Abaqus model with subroutine
	
	Returns
	-------
	ABQ_input_concrete_mech : .xls file
		 Results are written to workdir
	"""
	
	if irreversible == False:
		compressive_conc, elast_conc = MechanicalProperties.c_stress_strain_compression_T(fcm,aggregates=aggregates) # concrete compressive stress strain data
		tensile_conc = MechanicalProperties.c_stress_displacement_tension_T(fcm, fct = fct) # concrete tensile stress displacement data
		cet_conc = MechanicalProperties.c_thermal_expansion_coef_EC(delta=20) # concrete coefcient of thermal expansion
		aux = MechanicalProperties.Abaqus_CDP_parameters()
	
	if irreversible == True:
		print('not fully implemented! For now only irrecoverable thermal strain')
		cet_conc = MechanicalProperties.c_thermal_expansion_coef_EC(delta=20) # concrete coefcient of thermal expansion
	
	# generate output
	filename = 'output/ABQ_input_concrete_mech'
	out_list = [elast_conc,compressive_conc,tensile_conc,cet_conc,aux]
	sheet_list = ['c elastic','c compressive','c tensile','c thermal expansion','CDP parameters']
	admin.dfList_writeToExcel(out_list,filename,sheet_list)
	print('written Abaqus inputs for fc = {} MPa to {}.xlsx'.format(fcm,filename))


def abaqus_steel_properties_mech(fy=500,E=210000,irreversible=False):
	""" Create inputs for temperature dependant steel mechanical properties 
	(stress-strain curves, Young's modulus, thermal expansion) required in an Abaqus thermal-stress analysis
	
	Parameters
	----------
	fy : float (optional)
		Steel yield strength at 20°C [MPa]
	
	E : float (optional)
		Steel Young's modulus at 20°C [MPa]
	
	irreversible : boolean (optional) NOT IMPLEMENTED
		False = mechanical properties in the cooling phase are fully recovered (default)
		True =  mechanical properties in the cooling phase are irreversible, requires Abaqus model with subroutine
	
	Returns
	-------
	ABQ_input_concrete_mech : .xls file
		 Results are written to workdir
	"""
	
	if irreversible == False:
		stress_strain_steel, elast_steel = MechanicalProperties.s_stress_strain_EC_T(fy,E)
		cet_steel = MechanicalProperties.s_thermal_expansion_coef_EC()
	
	if irreversible == True:
		print('not implemented!')
	
	# generate output
	filename = 'output/ABQ_input_steel_mech'
	out_list = [stress_strain_steel, elast_steel,cet_steel]
	sheet_list = ['s stress-strain','s elastic','s thermal expansion']
	admin.dfList_writeToExcel(out_list,filename,sheet_list)
	print('written Abaqus inputs for fy = {} MPa to {}.xlsx'.format(fy,filename))


def abaqus_prestressingsteel_properties_mech(fp=1860,Ep=195000,irreversible=False):
	""" Create inputs for temperature dependant prestressing steel mechanical properties 
	(stress-strain curves, Young's modulus, thermal expansion) required in an Abaqus thermal-stress analysis
	
	Parameters
	----------
	fp : float (optional)
		Steel strength at 20°C [MPa]
	
	Ep : float (optional)
		Steel Young's modulus at 20°C [MPa]
	
	irreversible : boolean (optional) NOT IMPLEMENTED
		False = mechanical properties in the cooling phase are fully recovered (default)
		True =  mechanical properties in the cooling phase are irreversible, requires Abaqus model with subroutine
	
	Returns
	-------
	ABQ_input_concrete_mech : .xls file
		 Results are written to workdir
	"""
	
	if irreversible == False:
		stress_strain_p_steel, elast_p_steel = MechanicalProperties.p_stress_strain_EC_T(fp,Ep,softening=True)
		cet_p_steel = MechanicalProperties.p_thermal_expansion_coef_EC()
	
	if irreversible == True:
		print('not implemented!')
	
	# generate output
	filename = 'output/ABQ_input_prestressingsteel_mech'
	out_list = [stress_strain_p_steel, elast_p_steel,cet_p_steel]
	sheet_list = ['p elastic','p stress-strain','p thermal expansion']
	admin.dfList_writeToExcel(out_list,filename,sheet_list)
	print('written Abaqus inputs for fp = {} MPa to {}.xlsx'.format(fp,filename))


#########################
## STAND ALONE - DEBUG ##
#########################

if __name__ == "__main__":
	
	# TEST 1: Concrete mechanical properties
	abaqus_concrete_properties_th(irreversible=False,moisture=2)
	
	# TEST 2: Concrete mechanical properties
	abaqus_steel_properties_th()
	
	# TEST 3: Concrete mechanical properties
	fcm = 50 # [MPa]
	abaqus_concrete_properties_mech(fcm)
	
	# TEST 4: Steel mechanical properties
	fy = 500 # [MPa]
	E = 210000 # [MPa]
	abaqus_steel_properties_mech(fy,E)
	
	# TEST 5: Prestressing steel mechanical properties
	fp = 1860 # [MPa]
	Ep = 195000 # [MPa]
	abaqus_prestressingsteel_properties_mech(fp,Ep)


"""Thermal input file generation

Notes
-----
The Thermal module allows thermal input file generation for SAFIR.
"""

#############
## IMPORTS ##
#############

######################
## MODULE FUNCTIONS ##
######################

#########################
## STAND ALONE - DEBUG ##
#########################

## Test Function ##
###################

def Test(sentence='Default sentence'):
	""" Print sentence to screen - Test module activation

	Parameters
	----------
	sentence :	str
		input sentence

	Returns
	-------
	None

	Returns
	-------
	None

	Reference
	---------
	None

	Examples
	--------
	>>> import magnelPy.SAFIRshell as SAFIRshell
	>>> SAFIRshell.Thermal.Test()
	Default sentence
 	>>> SAFIRshell.Thermal.Test("Another test")
 	Another test
	"""
	print(sentence)

## Local testing ##
###################

if __name__ == "__main__":

	Test()
	Test('Another test')
	help(Test)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="magnelPy",
    version="0.2.4",
    author="magnelPy",
    author_email="ruben.vancoile@ugent.be",
    description="Python toolbox SFE team research group of Ghent University",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.ugent.be/MagnelPy/MagnelPy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

## release procedure ##
# update version in setup.py
# update release info in RELEASE.md
# upload to github repository
# >>> cd .\MagnelPy # location of setup.py
# python setup.py sdist bdist_wheel
# twine upload [-u USERNAME] [-p PASSWORD] dist/* 